@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python3
 
 # Lighting Example
 # Written in 2012, 2013, 2014 by Julian Marchant <onpon4@riseup.net>
@@ -22,7 +22,7 @@ class glob(object):
 
 class Game(sge.Game):
     def event_key_press(self, key, char):
-        if key == u'escape':
+        if key == 'escape':
             self.end()
 
     def event_close(self):
@@ -31,23 +31,23 @@ class Game(sge.Game):
 
 class Circle(sge.StellarClass):
     def __init__(self, x, y, player=0):
-        super(Circle, self).__init__(x, y, 5, sprite=u'circle',
+        super(Circle, self).__init__(x, y, 5, sprite='circle',
                                      collision_precise=True)
         self.player = player
 
     def event_create(self):
         self.image_alpha = 200
-        self.image_blend = u'blue'
+        self.image_blend = 'blue'
 
-        self.darkness = sge.StellarClass(0, 0, 10000, sprite=u'darkness',
+        self.darkness = sge.StellarClass(0, 0, 10000, sprite='darkness',
                                          detects_collisions=False)
         sge.game.current_room.add(self.darkness)
 
     def event_step(self, time_passed):
-        left_key = [u'left', u'a', u'j', u'kp_4'][self.player]
-        right_key = [u'right', u'd', u'l', u'kp_6'][self.player]
-        up_key = [u'up', u'w', u'i', u'kp_8'][self.player]
-        down_key = [u'down', u's', u'k', u'kp_5'][self.player]
+        left_key = ['left', 'a', 'j', 'kp_4'][self.player]
+        right_key = ['right', 'd', 'l', 'kp_6'][self.player]
+        up_key = ['up', 'w', 'i', 'kp_8'][self.player]
+        down_key = ['down', 's', 'k', 'kp_5'][self.player]
 
         self.xvelocity = (sge.get_key_pressed(right_key) -
                           sge.get_key_pressed(left_key))
@@ -66,8 +66,8 @@ class Circle(sge.StellarClass):
 
         # Light up part of the room
         glob.darkness_sprite.draw_rectangle(0, 0, sge.game.width,
-                                            sge.game.height, fill=u"black")
-        glob.darkness_sprite.draw_sprite(u'light', 0, self.x, self.y,
+                                            sge.game.height, fill="black")
+        glob.darkness_sprite.draw_sprite('light', 0, self.x, self.y,
                                          blend_mode=sge.BLEND_RGBA_SUBTRACT)
 
 def main():
@@ -75,16 +75,16 @@ def main():
     game = Game()
 
     # Load sprites
-    circle_sprite = sge.Sprite(u'circle', width=32, height=32, origin_x=16,
+    circle_sprite = sge.Sprite('circle', width=32, height=32, origin_x=16,
                                origin_y=16)
-    fence_sprite = sge.Sprite(u'fence')
-    light_sprite = sge.Sprite(u'light', width=128, height=128, origin_x=64,
+    fence_sprite = sge.Sprite('fence')
+    light_sprite = sge.Sprite('light', width=128, height=128, origin_x=64,
                               origin_y=64)
 
-    glob.darkness_sprite = sge.Sprite(ID=u'darkness', width=sge.game.width,
+    glob.darkness_sprite = sge.Sprite(ID='darkness', width=sge.game.width,
                                       height=sge.game.height)
     glob.darkness_sprite.draw_rectangle(0, 0, sge.game.width, sge.game.height,
-                                        fill=u"black")
+                                        fill="black")
 
     # Load backgrounds
     layers = (sge.BackgroundLayer(fence_sprite, 0, 380, 0, yrepeat=False),)
@@ -103,5 +103,5 @@ def main():
     game.start()
 
 
-if __name__ == u'__main__':
+if __name__ == '__main__':
     main()
