@@ -13,20 +13,25 @@
 # <http://creativecommons.org/publicdomain/zero/1.0/>.
 
 
-u"""Stellar Game Engine example
+"""Stellar Game Engine example
 
 This is a simple example of a possible game in Stellar Game Engine, just
 to give a general idea of how it will be used.
 
 """
 
+from __future__ import division
+from __future__ import absolute_import
+from __future__ import print_function
+from __future__ import unicode_literals
+
 import sge
 import random
 
 
 class Game(sge.Game):
     def event_key_press(self, key, char):
-        if key == u'escape':
+        if key == 'escape':
             self.end()
 
     def event_close(self):
@@ -35,23 +40,27 @@ class Game(sge.Game):
 
 class Circle(sge.StellarClass):
     def __init__(self, x, y):
-        super(Circle, self).__init__(x, y, 5, sprite=u'rotator',
+        super(Circle, self).__init__(x, y, 5, sprite='rotator',
                                      collision_precise=True)
 
     def event_create(self):
         self.image_alpha = 200
         if self.collides(sge.game.mouse):
-            self.image_blend = u'#ff0000'
+            self.image_blend = '#ff0000'
         else:
-            self.image_blend = u'blue'
+            self.image_blend = 'blue'
+
+        if random.random() < 0.5:
+            self.image_xscale = 2
+            self.image_yscale = 2
 
     def event_step(self, time_passed):
         self.image_rotation += 2
         sge.game.current_room.project_circle(self.x, self.y, self.z + 1, 8,
-                                             outline=u"green")
+                                             outline="green")
 
         if self.collides(sge.game.mouse):
-            self.image_blend = u"red"
+            self.image_blend = "red"
         else:
             self.image_blend = (0, 0, 255)
 
@@ -61,8 +70,8 @@ def main():
     game = Game(delta=True)
 
     # Load sprites
-    sge.Sprite(u'rotator')
-    fence_sprite = sge.Sprite(u'fence')
+    sge.Sprite('rotator')
+    fence_sprite = sge.Sprite('fence')
 
     # Load backgrounds
     layers = (sge.BackgroundLayer(fence_sprite, 0, 380, 0, yrepeat=False),)
@@ -84,5 +93,5 @@ def main():
     game.start()
 
 
-if __name__ == u'__main__':
+if __name__ == '__main__':
     main()
