@@ -1,4 +1,4 @@
-# Copyright (C) 2012, 2013 Julian Marchant <onpon4@riseup.net>
+# Copyright (C) 2012, 2013, 2014 Julian Marchant <onpon4@riseup.net>
 # 
 # This file is part of the Pygame SGE.
 # 
@@ -15,22 +15,17 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with the Pygame SGE.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import division
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import pygame
 
 import sge
 
 
-__all__ = [u'Background']
+__all__ = ['Background']
 
 
-class Background(object):
+class Background:
 
-    u"""Background class.
+    """Background class.
 
     This class stores the layers that make up the background (which
     contain the information about what images to use and where) as well
@@ -53,7 +48,7 @@ class Background(object):
     """
 
     def __init__(self, layers, color, ID=None):
-        u"""Constructor method.
+        """Constructor method.
 
         Arguments:
 
@@ -96,6 +91,10 @@ class Background(object):
 
         self.layers = tuple(sorted_layers)
 
+    def destroy(self):
+        """Destroy the background."""
+        del sge.game.backgrounds[self.id]
+
     def _get_background(self):
         # Return the static background this frame.
         background = pygame.Surface((round(sge.game.width * sge.game._xscale),
