@@ -1,4 +1,4 @@
-# Copyright (C) 2012, 2013 Julian Marchant <onpon4@riseup.net>
+# Copyright (C) 2012, 2013, 2014 Julian Marchant <onpon4@riseup.net>
 # 
 # This file is part of the Pygame SGE.
 # 
@@ -15,20 +15,15 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with the Pygame SGE.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import division
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import sge
 
 
-__all__ = [u'BackgroundLayer']
+__all__ = ['BackgroundLayer']
 
 
-class BackgroundLayer(object):
+class BackgroundLayer:
 
-    u"""Special class used for background layers.
+    """Special class used for background layers.
 
     This class stores a sprite and certain information for a layer of a
     background.  In particular, it stores the location of the layer,
@@ -161,7 +156,7 @@ class BackgroundLayer(object):
 
     def __init__(self, sprite, x, y, z, ID=None, xscroll_rate=1,
                  yscroll_rate=1, xrepeat=True, yrepeat=True):
-        u"""Constructor method.
+        """Constructor method.
 
         Arguments:
 
@@ -191,7 +186,7 @@ class BackgroundLayer(object):
             self.id = self.sprite.id
 
             while self.id in sge.game.background_layers:
-                self.id += u"_"
+                self.id += "_"
 
         self._image_index = 0
         self._count = 0
@@ -207,6 +202,10 @@ class BackgroundLayer(object):
 
         sge.game.background_layers[self.id] = self
 
+    def destroy(self):
+        """Destroy the background layer."""
+        del sge.game.background_layers[self.id]
+
     def _update(self, time_passed):
         # Update the animation frame.
         if self._frame_time is not None:
