@@ -16,6 +16,7 @@
 # along with the Pygame SGE.  If not, see <http://www.gnu.org/licenses/>.
 
 import os
+import warnings
 
 import pygame
 
@@ -25,7 +26,7 @@ import sge
 __all__ = ['Game']
 
 
-class Game(object):
+class Game:
 
     """Class which handles the game.
 
@@ -326,6 +327,7 @@ class Game(object):
         self._js_names = {}
         self._js_ids = {}
         self._pygame_sprites = pygame.sprite.LayeredDirty()
+        self._window_projections = []
         self.mouse = sge.Mouse()
 
         # Setup sound channels
@@ -363,6 +365,13 @@ class Game(object):
         self._object_start_collision_ellipse = {}
         self._object_start_collision_precise = {}
 
+        self._dot_cache = {}
+        self._line_cache = {}
+        self._rectangle_cache = {}
+        self._ellipse_cache = {}
+        self._circle_cache = {}
+        self._text_cache = {}
+
     def start(self):
         """Start the game at the first room.
 
@@ -418,11 +427,11 @@ class Game(object):
                 self._object_start_collision_ellipse[obj.id] = obj.collision_ellipse
                 self._object_start_collision_precise[obj.id] = obj.collision_precise
 
-            display_surface = self._window.copy()
+            self._display_surface = self._window.copy()
 
             self.rooms[0].start()
             background = None
-            mouse_previous_rect = pygame.Rect(0, 0, 1, 1)
+            cleanup_rects = []
             numviews = 0
             _fps_time = 0
             self._clock.tick()
@@ -577,12 +586,10 @@ class Game(object):
                     elif event.type == pygame.VIDEORESIZE:
                         if sge.DEBUG:
                             print('Video resize detected.')
-                        self._window.blit(display_surface, (0, 0))
+                        self._window.blit(self._display_surface, (0, 0))
                         self._window_width = event.w
                         self._window_height = event.h
                         self._set_mode()
-                        display_surface = self._window.copy()
-                        self._background_changed = True
 
                 real_time_passed = self._clock.tick(self.fps)
 
@@ -597,7 +604,7 @@ class Game(object):
                     _fps_time += real_time_passed
                     if _fps_time >= 250:
                         _fps_time = 0
-                        self.window_text = "FPS: {0}; Delta: {1}".format(
+                        self.window_text = "FPS: {}; Delta: {}".format(
                             int(1000 / real_time_passed), delta_mult)
 
                 # Step events
@@ -617,6 +624,24 @@ class Game(object):
                     else:
                         obj.event_inactive_step(real_time_passed)
 
+                # Set objects' colliders
+                room = self.current_room
+                for ref in self._colliders:
+                    obj = ref()
+                    if obj is not None:
+                        obj._colliders = []
+                        for area in obj._collision_areas:
+                            if area is not None:
+                                i, j = area
+                                room_area = room._collision_areas[i][j]
+                            else:
+                                room_area = room._collision_area_void
+
+                            for other in room_area:
+                                if (other is not obj and
+                                        other not in obj._colliders):
+                                    obj._colliders.append(other)
+
                 # Detect collisions
                 for ref in self._colliders:
                     obj = ref()
@@ -663,28 +688,37 @@ class Game(object):
 
                 # Redraw
                 if self._background_changed or background is None:
-                    w = max(1, display_surface.get_width())
-                    h = max(1, display_surface.get_height())
+                    w = max(1, self._display_surface.get_width())
+                    h = max(1, self._display_surface.get_height())
                     background = pygame.Surface((w, h))
                     b = self.current_room.background._get_background()
                     background.blit(b, (self._x, self._y))
-                    display_surface.blit(background, (0, 0))
+                    self._display_surface.blit(background, (0, 0))
                     self._background_changed = False
-                    self._pygame_sprites.clear(display_surface, background)
+                    self._pygame_sprites.clear(self._display_surface,
+                                               background)
                     for sprite in self._pygame_sprites:
                         sprite.rect = pygame.Rect(0, 0, 1, 1)
                         sprite.image = pygame.Surface((1, 1))
                         sprite.image.set_colorkey((0, 0, 0))
                     self._pygame_sprites.update()
-                    self._pygame_sprites.draw(display_surface)
-                    dirty = [display_surface.get_rect()]
+                    self._pygame_sprites.draw(self._display_surface)
+                    dirty = [self._display_surface.get_rect()]
                 else:
-                    self._pygame_sprites.clear(display_surface, background)
+                    self._pygame_sprites.clear(self._display_surface,
+                                               background)
                     self._pygame_sprites.update()
-                    dirty = self._pygame_sprites.draw(display_surface)
+                    dirty = self._pygame_sprites.draw(self._display_surface)
 
-                self._window.blit(display_surface, (0, 0))
+                self._window.blit(self._display_surface, (0, 0))
 
+                # Window projections
+                self.mouse.project_cursor()
+                dirty.extend(cleanup_rects)
+                cleanup_rects = self._show_projections()
+                dirty.extend(cleanup_rects)
+
+                # Letterbox/pillarbox
                 top_bar = pygame.Rect(0, 0, w, self._y)
                 bottom_bar = pygame.Rect(0, h - self._y, w, self._y)
                 left_bar = pygame.Rect(0, 0, self._x, h)
@@ -702,22 +736,6 @@ class Game(object):
                     self._window.fill((0, 0, 0), right_bar)
                     dirty.append(right_bar)
 
-                if (not self.grab_input and self.mouse.visible and
-                        self.mouse.sprite is not None):
-                    mx, my = pygame.mouse.get_pos()
-                    mouse_rect = pygame.Rect(
-                        mx - self.mouse.sprite.origin_x,
-                        my - self.mouse.sprite.origin_y,
-                        self.mouse.sprite.width, self.mouse.sprite.height)
-                    img = self.mouse.sprite._get_image(
-                        self.mouse.image_index, self.mouse.image_xscale,
-                        self.mouse.image_yscale, self.mouse.image_rotation,
-                        self.mouse.image_alpha, self.mouse.image_blend)
-                    self._window.blit(img, mouse_rect)
-                    dirty.append(mouse_rect)
-                    dirty.append(mouse_previous_rect)
-                    mouse_previous_rect = mouse_rect
-
                 if sge.hardware_rendering:
                     pygame.display.flip()
                 else:
@@ -893,7 +911,6 @@ class Game(object):
                     self._window_width = event.w
                     self._window_height = event.h
                     self._set_mode()
-                    self._background_changed = True
                     screenshot = pygame.transform.scale(orig_screenshot,
                                                         (event.w, event.h))
                     background = pygame.transform.scale(orig_background,
@@ -905,18 +922,8 @@ class Game(object):
             # Redraw
             self._window.blit(background, (0, 0))
 
-            if (not self.grab_input and self.mouse.visible and
-                    self.mouse.sprite is not None):
-                mx, my = pygame.mouse.get_pos()
-                mouse_rect = pygame.Rect(
-                    mx - self.mouse.sprite.origin_x,
-                    my - self.mouse.sprite.origin_y,
-                    self.mouse.sprite.width, self.mouse.sprite.height)
-                img = self.mouse.sprite._get_image(
-                    self.mouse.image_index, self.mouse.image_xscale,
-                    self.mouse.image_yscale, self.mouse.image_rotation,
-                    self.mouse.image_alpha, self.mouse.image_blend)
-                self._window.blit(img, mouse_rect)
+            self.mouse.project_cursor()
+            self._show_projections()
 
             pygame.display.flip()
 
@@ -946,6 +953,155 @@ class Game(object):
                     if isinstance(obj, cls):
                         room.objects_by_class[cls].append(obj)
 
+    def project_dot(self, x, y, color):
+        """Project a single-pixel dot onto the game window.
+
+        Arguments:
+
+        - ``x`` -- The horizontal location relative to the window to
+          project the dot.
+        - ``y`` -- The vertical location relative to the window to
+          project the dot.
+
+        See the documentation for :meth:`sge.Sprite.draw_dot` for more
+        information.
+
+        """
+        sprite = self._get_dot_sprite(color)
+        self.project_sprite(sprite, 0, x, y)
+
+    def project_line(self, x1, y1, x2, y2, color, thickness=1,
+                     anti_alias=False):
+        """Project a line segment onto the game window.
+
+        Arguments:
+
+        - ``x1`` -- The horizontal location relative to the window of
+          the first endpoint of the projected line segment.
+        - ``y1`` -- The vertical location relative to the window of the
+          first endpoint of the projected line segment.
+        - ``x2`` -- The horizontal location relative to the window of
+          the second endpoint of the projected line segment.
+        - ``y2`` -- The vertical location relative to the window of the
+          second endpoint of the projected line segment.
+
+        See the documentation for :meth:`sge.Sprite.draw_line` for more
+        information.
+
+        """
+        thickness = abs(thickness)
+        x = min(x1, x2) - thickness // 2
+        y = min(y1, y2) - thickness // 2
+        x1 -= x
+        y1 -= y
+        x2 -= x
+        y2 -= y
+
+        sprite = self._get_line_sprite(x1, y1, x2, y2, color, thickness,
+                                       anti_alias)
+        self.project_sprite(sprite, 0, x, y)
+
+    def project_rectangle(self, x, y, width, height, fill=None, outline=None,
+                          outline_thickness=1):
+        """Project a rectangle onto the game window.
+
+        Arguments:
+
+        - ``x`` -- The horizontal location relative to the window to
+          project the rectangle.
+        - ``y`` -- The vertical location relative to the window to
+          project the rectangle.
+
+        See the documentation for :meth:`sge.Sprite.draw_rectangle` for
+        more information.
+
+        """
+        sprite = self._get_rectangle_sprite(width, height, fill, outline,
+                                            outline_thickness)
+        self.project_sprite(sprite, 0, x, y)
+
+    def project_ellipse(self, x, y, width, height, fill=None, outline=None,
+                        outline_thickness=1, anti_alias=False):
+        """Project an ellipse onto the game window.
+
+        Arguments:
+
+        - ``x`` -- The horizontal location relative to the window to
+          position the imaginary rectangle containing the ellipse.
+        - ``y`` -- The vertical location relative to the window to
+          position the imaginary rectangle containing the ellipse.
+        - ``width`` -- The width of the ellipse.
+        - ``height`` -- The height of the ellipse.
+        - ``fill`` -- The color of the fill of the ellipse.
+        - ``outline`` -- The color of the outline of the ellipse.
+        - ``outline_thickness`` -- The thickness of the outline of the
+          ellipse.
+        - ``anti_alias`` -- Whether or not anti-aliasing should be used.
+
+        See the documentation for :meth:`sge.Sprite.draw_ellipse` for
+        more information.
+
+        """
+        sprite = self._get_ellipse_sprite(width, height, fill, outline,
+                                          outline_thickness, anti_alias)
+        self.project_sprite(sprite, 0, x, y)
+
+    def project_circle(self, x, y, radius, fill=None, outline=None,
+                       outline_thickness=1, anti_alias=False):
+        """Project a circle onto the game window.
+
+        Arguments:
+
+        - ``x`` -- The horizontal location relative to the window to
+          position the center of the circle.
+        - ``y`` -- The vertical location relative to the window to
+          position the center of the circle.
+
+        See the documentation for :meth:`sge.Sprite.draw_circle` for
+        more information.
+
+        """
+        sprite = self._get_circle_sprite(radius, fill, outline,
+                                         outline_thickness, anti_alias)
+        self.project_sprite(sprite, 0, x - radius, y - radius)
+
+    def project_sprite(self, sprite, image, x, y, blend_mode=None):
+        """Project a sprite onto the game window.
+
+        Arguments:
+
+        - ``x`` -- The horizontal location relative to the window to
+          project ``sprite``.
+        - ``y`` -- The vertical location relative to the window to
+          project ``sprite``.
+
+        See the documentation for :meth:`sge.Sprite.draw_sprite` for
+        more information.
+
+        """
+        img = sprite._get_image(image)
+        self._window_projections.append((img, x, y, blend_mode))
+
+    def project_text(self, font, text, x, y, width=None, height=None,
+                    color="black", halign=sge.ALIGN_LEFT, valign=sge.ALIGN_TOP,
+                    anti_alias=True):
+        """Project text onto the game window.
+
+        Arguments:
+
+        - ``x`` -- The horizontal location relative to the window to
+          project the text.
+        - ``y`` -- The vertical location relative to the window to
+          project the text.
+
+        See the documentation for :meth:`sge.Sprite.draw_text` for more
+        information.
+
+        """
+        sprite = self._get_text_sprite(font, text, width, height, color,
+                                       halign, valign, anti_alias)
+        self.project_sprite(sprite, 0, x, y)
+
     def event_game_start(self):
         """Game start event.
 
@@ -1426,11 +1582,24 @@ class Game(object):
             if sge.hardware_rendering:
                 flags |= pygame.HWSURFACE | pygame.DOUBLEBUF
 
-            self._window = pygame.display.set_mode((0, 0), flags)
+            modes = pygame.display.list_modes()
+            if modes != -1 and modes:
+                self._window_width, self._window_height = modes[0]
+            else:
+                w = "Couldn't find out the maximum resolution! Assuming 640x480."
+                warnings.warn(w)
+                self._window_width = 640
+                self._window_height = 480
+
+            self._background_changed = True
+            self._window = pygame.display.set_mode(
+                (self._window_width, self._window_height), flags)
 
             if not self.scale:
-                self._xscale = info.current_w / self.width
-                self._yscale = info.current_h / self.height
+                ##self._xscale = info.current_w / self.width
+                ##self._yscale = info.current_h / self.height
+                self._xscale = self._window_width / self.width
+                self._yscale = self._window_height / self.height
 
                 if self.scale_proportional:
                     self._xscale = min(self._xscale, self._yscale)
@@ -1438,10 +1607,8 @@ class Game(object):
 
             w = max(1, self._window.get_width())
             h = max(1, self._window.get_height())
-            self._x = int(round((w - int(round(self.width * self._xscale))) /
-                                2))
-            self._y = int(round((h - int(round(self.height * self._yscale))) /
-                                2))
+            self._x = int(round((w - self.width * self._xscale) / 2))
+            self._y = int(round((h - self.height * self._yscale) / 2))
         else:
             self._x = 0
             self._y = 0
@@ -1463,14 +1630,137 @@ class Game(object):
 
             w = max(1, self._window.get_width())
             h = max(1, self._window.get_height())
-            self._x = int(round((w - int(round(self.width * self._xscale))) /
-                                2))
-            self._y = int(round((h - int(round(self.height * self._yscale))) /
-                                2))
+            self._x = int(round((w - self.width * self._xscale) / 2))
+            self._y = int(round((h - self.height * self._yscale) / 2))
 
-        # Refresh sprites
-        for s in self.sprites:
-            self.sprites[s]._refresh()
+        self._display_surface = self._window.copy()
+        self._background_changed = True
+
+    def _get_dot_sprite(self, color):
+        # Return a sprite for the given dot.
+        i = (color,)
+        if i in self._dot_cache:
+            sprite = self._dot_cache[i]
+        else:
+            sprite = sge.Sprite(None, width=1, height=1)
+            sprite.draw_dot(0, 0, color)
+            self._dot_cache[i] = sprite
+            sprite.destroy()
+
+        return sprite
+
+    def _get_line_sprite(self, x1, y1, x2, y2, color, thickness, anti_alias):
+        # Return a sprite for the given line.
+        w = int(round(abs(x2 - x1) + thickness))
+        h = int(round(abs(y2 - y1) + thickness))
+        i = (x1, y1, x2, y2, color, thickness, anti_alias)
+        if i in self._line_cache:
+            sprite = self._line_cache[i]
+        else:
+            sprite = sge.Sprite(None, width=w, height=h)
+            sprite.draw_line(x1, y1, x2, y2, color, thickness, anti_alias)
+            self._line_cache[i] = sprite
+            sprite.destroy()
+
+        return sprite
+
+    def _get_rectangle_sprite(self, width, height, fill, outline,
+                              outline_thickness):
+        # Return a sprite for the given rectangle.
+        i = (width, height, fill, outline, outline_thickness)
+        if i in self._rectangle_cache:
+            sprite = self._rectangle_cache[i]
+        else:
+            outline_thickness = abs(outline_thickness)
+            draw_x = outline_thickness // 2
+            draw_y = outline_thickness // 2
+            x -= draw_x
+            y -= draw_y
+            w = width + outline_thickness
+            h = height + outline_thickness
+            sprite = sge.Sprite(None, width=w, height=h)
+            sprite.draw_rectangle(draw_x, draw_y, w, h, fill, outline,
+                                  outline_thickness)
+            self._rectangle_cache[i] = sprite
+            sprite.destroy()
+
+        return sprite
+
+    def _get_ellipse_sprite(self, width, height, fill, outline,
+                            outline_thickness, anti_alias):
+        # Return a sprite for the given ellipse.
+        i = (width, height, fill, outline, outline_thickness, anti_alias)
+        if i in self._ellipse_cache:
+            sprite = self._ellipse_cache[i]
+        else:
+            outline_thickness = abs(outline_thickness)
+            draw_x = outline_thickness // 2
+            draw_y = outline_thickness // 2
+            x -= draw_x
+            y -= draw_y
+            w = width + outline_thickness
+            h = height + outline_thickness
+            sprite = sge.Sprite(None, width=w, height=h)
+            sprite.draw_ellipse(draw_x, draw_y, w, h, fill, outline,
+                                outline_thickness)
+            self._ellipse_cache[i] = sprite
+            sprite.destroy()
+
+        return sprite
+
+    def _get_circle_sprite(self, radius, fill, outline, outline_thickness,
+                           anti_alias):
+        # Return a sprite for the given circle.
+        i = (radius, fill, outline, outline_thickness, anti_alias)
+        if i in self._circle_cache:
+            sprite = self._circle_cache[i]
+        else:
+            outline_thickness = abs(outline_thickness)
+            xy = radius + outline_thickness // 2
+            wh = 2 * radius + outline_thickness
+            sprite = sge.Sprite(None, width=wh, height=wh)
+            sprite.draw_circle(xy, xy, radius, fill, outline, outline_thickness,
+                               anti_alias)
+            self._circle_cache[i] = sprite
+            sprite.destroy()
+
+        return sprite
+
+    def _get_text_sprite(self, font, text, width, height, color, halign,
+                         valign, anti_alias):
+        # Return a sprite for the given text.
+        i = (font, text, width, height, color, halign, valign, anti_alias)
+        if i in sge.game._text_cache:
+            sprite = sge.game._text_cache[i]
+        else:
+            if not isinstance(font, sge.Font):
+                font = sge.game.fonts[font]
+
+            w, h = font.get_size(text, width, height)
+            draw_x = {sge.ALIGN_LEFT: 0, sge.ALIGN_CENTER: w / 2,
+                      sge.ALIGN_RIGHT: w}.setdefault(halign, w / 2)
+            draw_y = {sge.ALIGN_TOP: 0, sge.ALIGN_MIDDLE: h / 2,
+                      sge.ALIGN_BOTTOM: h}.setdefault(valign, h / 2)
+            sprite = sge.Sprite(None, width=w, height=h)
+            sprite.draw_text(font, text, draw_x, draw_y, width, height, color,
+                             halign, valign, anti_alias)
+            sge.game._text_cache[i] = sprite
+            sprite.destroy()
+
+        return sprite
+
+    def _show_projections(self):
+        # Show the window projections and return the area rects list.
+        rects = []
+        for projection in self._window_projections:
+            (image, x, y, blend_mode) = projection
+            rect = image.get_rect(left=(self._x + x * self._xscale),
+                                  top=(self._y + y * self._yscale))
+            self._window.blit(image, rect)
+            rects.append(rect)
+
+        self._window_projections = []
+        return rects
 
     def _get_channel(self):
         # Return a channel for a sound effect to use.
