@@ -20,15 +20,17 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
+import pygame
+
 import sge
 
 
-__all__ = [u'Room']
+__all__ = ['Room']
 
 
 class Room(object):
 
-    u"""Class for rooms.
+    """Class for rooms.
 
     This class stores the settings and objects found in a room.  Rooms
     are used to create separate parts of the game, such as levels and
@@ -79,14 +81,14 @@ class Room(object):
     .. attribute:: room_number
 
        The index of this room in the game, where ``0`` is the first
-       room, or :const:`None` if this room has not been added to a game.
-       (Read-only)
+       room.  (Read-only)
 
     """
 
     def __init__(self, objects=(), width=None, height=None, views=None,
-                 background=None, background_x=0, background_y=0):
-        u"""Constructor method.
+                 background=None, background_x=0, background_y=0,
+                 room_number=None):
+        """Constructor method.
 
         Arguments:
 
@@ -97,6 +99,9 @@ class Room(object):
         - ``background`` -- The :class:`sge.Background` object used.  If
           set to :const:`None`, a new background will be created with no
           layers and the color set to ``"black"``.
+        - ``room_number`` -- The position in :data:`sge.game.rooms` to
+          insert this room into.  If set to :const:`None`, it will be
+          appended to the end of the list.
 
         All other arguments set the respective initial attributes of the
         room.  See the documentation for :class:`sge.Room` for more
@@ -126,11 +131,15 @@ class Room(object):
         if background is not None:
             self.background = background
         else:
-            self.background = sge.Background((), u'black')
+            self.background = sge.Background((), 'black')
         self._start_background = self.background
 
-        self.room_number = len(sge.game.rooms)
-        sge.game.rooms.append(self)
+        if room_number is None:
+            self.room_number = len(sge.game.rooms)
+            sge.game.rooms.append(self)
+        else:
+            self.room_number = room_number
+            sge.game.rooms.insert(room_number, self)
 
         self._started = False
         self._has_started = False
@@ -158,8 +167,26 @@ class Room(object):
         self._object_start_collision_ellipse = {}
         self._object_start_collision_precise = {}
 
+        if self.views:
+            self._collision_area_size = max(int(self.views[0].width / 10),
+                                            int(self.views[0].height / 10))
+        else:
+            self._collision_area_size = sge.COLLISION_AREA_SIZE_DEFAULT
+
+        size = self._collision_area_size
+        self._collision_areas = []
+        for i in xrange(0, self.width, size):
+            column = [[] for j in xrange(0, self.height, size)]
+            self._collision_areas.append(column)
+
+        # The "Void" is the area outside the room.  This area is
+        # infinite, so everything in the Void is checked against
+        # everything else in the Void (otherwise an infinite grid, which
+        # is impossible, would be needed).
+        self._collision_area_void = []
+
     def add(self, obj):
-        u"""Add a StellarClass object to the room.
+        """Add a StellarClass object to the room.
 
         Arguments:
 
@@ -182,7 +209,7 @@ class Room(object):
                     obj.event_create()
 
     def start(self):
-        u"""Start the room.
+        """Start the room.
 
         If the room has been changed, reset it to its original state.
 
@@ -193,7 +220,7 @@ class Room(object):
         self.resume()
 
     def resume(self):
-        u"""Continue the room from where it left off.
+        """Continue the room from where it left off.
 
         If the room is unchanged (e.g. has not been started yet), this
         method behaves in the same way that :meth:`sge.Room.start` does.
@@ -243,7 +270,7 @@ class Room(object):
         self._has_started = True
 
     def end(self, next_room=None, resume=True):
-        u"""End the current room.
+        """End the current room.
 
         Arguments:
 
@@ -275,7 +302,7 @@ class Room(object):
             sge.game.end()
 
     def project_dot(self, x, y, z, color):
-        u"""Project a single-pixel dot onto the room.
+        """Project a single-pixel dot onto the room.
 
         Arguments:
 
@@ -289,14 +316,12 @@ class Room(object):
         information.
 
         """
-        sprite = sge.Sprite(None, width=1, height=1)
-        sprite.draw_dot(0, 0, color)
-        p = _Projection(x, y, z, sprite=sprite, detects_collisions=False)
-        self.add(p)
+        sprite = sge.game._get_dot_sprite(color)
+        self.project_sprite(sprite, 0, x, y, z)
 
     def project_line(self, x1, y1, x2, y2, z, color, thickness=1,
                      anti_alias=False):
-        u"""Project a line segment onto the room.
+        """Project a line segment onto the room.
 
         Arguments:
 
@@ -317,20 +342,18 @@ class Room(object):
         thickness = abs(thickness)
         x = min(x1, x2) - thickness // 2
         y = min(y1, y2) - thickness // 2
-        w = abs(x2 - x1) + thickness
-        h = abs(y2 - y1) + thickness
         x1 -= x
         y1 -= y
         x2 -= x
         y2 -= y
-        sprite = sge.Sprite(None, width=w, height=h)
-        sprite.draw_line(x1, y1, x2, y2, color, thickness, anti_alias)
-        p = _Projection(x, y, z, sprite=sprite, detects_collisions=False)
-        self.add(p)
+
+        sprite = sge.game._get_line_sprite(x1, y1, x2, y2, color, thickness,
+                                           anti_alias)
+        self.project_sprite(sprite, 0, x, y, z)
 
     def project_rectangle(self, x, y, z, width, height, fill=None,
                           outline=None, outline_thickness=1):
-        u"""Project a rectangle onto the room.
+        """Project a rectangle onto the room.
 
         Arguments:
 
@@ -344,22 +367,13 @@ class Room(object):
         more information.
 
         """
-        outline_thickness = abs(outline_thickness)
-        draw_x = outline_thickness // 2
-        draw_y = outline_thickness // 2
-        x -= draw_x
-        y -= draw_y
-        w = width + outline_thickness
-        h = height + outline_thickness
-        sprite = sge.Sprite(None, width=w, height=h)
-        sprite.draw_rectangle(draw_x, draw_y, w, h, fill, outline,
-                              outline_thickness)
-        p = _Projection(x, y, z, sprite=sprite, detects_collisions=False)
-        self.add(p)
+        sprite = sge.game._get_rectangle_sprite(width, height, fill, outline,
+                                                outline_thickness)
+        self.project_sprite(sprite, 0, x, y, z)
 
     def project_ellipse(self, x, y, z, width, height, fill=None,
                         outline=None, outline_thickness=1, anti_alias=False):
-        u"""Project an ellipse onto the room.
+        """Project an ellipse onto the room.
 
         Arguments:
 
@@ -380,22 +394,13 @@ class Room(object):
         more information.
 
         """
-        outline_thickness = abs(outline_thickness)
-        draw_x = outline_thickness // 2
-        draw_y = outline_thickness // 2
-        x -= draw_x
-        y -= draw_y
-        w = width + outline_thickness
-        h = height + outline_thickness
-        sprite = sge.Sprite(None, width=w, height=h)
-        sprite.draw_ellipse(draw_x, draw_y, w, h, fill, outline,
-                            outline_thickness)
-        p = _Projection(x, y, z, sprite=sprite, detects_collisions=False)
-        self.add(p)
+        sprite = sge.game._get_ellipse_sprite(width, height, fill, outline,
+                                              outline_thickness, anti_alias)
+        self.project_sprite(sprite, 0, x, y, z)
 
     def project_circle(self, x, y, z, radius, fill=None, outline=None,
                        outline_thickness=1, anti_alias=False):
-        u"""Project a circle onto the room.
+        """Project a circle onto the room.
 
         Arguments:
 
@@ -409,18 +414,12 @@ class Room(object):
         more information.
 
         """
-        outline_thickness = abs(outline_thickness)
-        xy = radius + outline_thickness // 2
-        wh = 2 * radius + outline_thickness
-        sprite = sge.Sprite(None, width=wh, height=wh)
-        sprite.draw_circle(xy, xy, radius, fill, outline, outline_thickness,
-                           anti_alias)
-        p = _Projection(x - radius, y - radius, z, sprite=sprite,
-                        detects_collisions=False)
-        self.add(p)
+        sprite = sge.game._get_circle_sprite(radius, fill, outline,
+                                             outline_thickness, anti_alias)
+        self.project_sprite(sprite, 0, x - radius, y - radius, z)
 
     def project_sprite(self, sprite, image, x, y, z, blend_mode=None):
-        u"""Project a sprite onto the room.
+        """Project a sprite onto the room.
 
         Arguments:
 
@@ -434,8 +433,8 @@ class Room(object):
         more information.
 
         """
-        if blend_mode & sge.BLEND_SCREEN:
-            w = u"Screen blend mode not supported. Normal blending used instead."
+        if blend_mode is not None and blend_mode & sge.BLEND_SCREEN:
+            w = "Screen blend mode not supported. Normal blending used instead."
             warnings.warn(w)
 
         pygame_flags = {
@@ -451,15 +450,14 @@ class Room(object):
             sge.BLEND_RGB_MAXIMUM: pygame.BLEND_RGB_MAX
             }.setdefault(blend_mode, 0)
 
-        p = _Projection(x, y, z, sprite=sprite, detects_collisions=False,
-                        image_index=image, image_fps=0)
-        p._pygame_sprite.blendmode = pygame_flags
-        self.add(p)
+        p = sge._PygameProjectionSprite(x, y, z, sprite, image)
+        p.blendmode = pygame_flags
+        sge.game._pygame_sprites.add(p, layer=z)
 
     def project_text(self, font, text, x, y, z, width=None, height=None,
-                    color=u"black", halign=sge.ALIGN_LEFT, valign=sge.ALIGN_TOP,
+                    color="black", halign=sge.ALIGN_LEFT, valign=sge.ALIGN_TOP,
                     anti_alias=True):
-        u"""Project text onto the room.
+        """Project text onto the room.
 
         Arguments:
 
@@ -473,22 +471,36 @@ class Room(object):
         information.
 
         """
-        if not isinstance(font, sge.Font):
-            font = sge.game.fonts[font]
+        sprite = sge.game._get_text_sprite(font, text, width, height, color,
+                                           halign, valign, anti_alias)
+        self.project_sprite(sprite, 0, x, y, z)
 
-        w, h = font.get_size(text, width, height)
-        draw_x = {sge.ALIGN_LEFT: 0, sge.ALIGN_CENTER: w / 2,
-                  sge.ALIGN_RIGHT: w}.setdefault(halign, w / 2)
-        draw_y = {sge.ALIGN_TOP: 0, sge.ALIGN_MIDDLE: h / 2,
-                  sge.ALIGN_BOTTOM: h}.setdefault(valign, h / 2)
-        sprite = sge.Sprite(None, width=w, height=h)
-        sprite.draw_text(font, text, draw_x, draw_y, width, height, color,
-                         halign, valign, anti_alias)
-        p = _Projection(x, y, z, sprite=sprite, detects_collisions=False)
-        self.add(p)
+    def move(self, room_number):
+        """Move the room.
+
+        Arguments:
+
+        - ``room_number`` -- The new position in :data:`sge.game.rooms` to
+          insert this room into.
+
+        """
+        del sge.game.rooms[self.room_number]
+        self.room_number = room_number
+        sge.game.rooms.insert(room_number, self)
+
+    def destroy(self):
+        """Destroy the room.
+
+        .. note::
+
+           If the room is being used, it will not be completely
+           destroyed until this use stops.
+
+        """
+        del sge.game.rooms[self.room_number]
 
     def event_room_start(self):
-        u"""Room start event.
+        """Room start event.
 
         Called when the room starts.  It is always called after any game
         start events and before any object create events occurring at
@@ -498,7 +510,7 @@ class Room(object):
         pass
 
     def event_room_resume(self):
-        u"""Room resume event.
+        """Room resume event.
 
         Called when the room resumes without being reset to its original
         state (i.e. via :meth:`sge.Room.resume`).
@@ -509,7 +521,7 @@ class Room(object):
         self.event_room_start()
 
     def event_room_end(self):
-        u"""Room end event.
+        """Room end event.
 
         Called when the room ends.  It is always called before any game
         end events occurring at the same time.
@@ -518,7 +530,7 @@ class Room(object):
         pass
 
     def event_step(self, time_passed):
-        u"""Room step event.
+        """Room step event.
 
         See the documentation for :meth:`sge.Game.event_step` for more
         information.
@@ -527,7 +539,7 @@ class Room(object):
         pass
 
     def event_key_press(self, key, char):
-        u"""Key press event.
+        """Key press event.
 
         See the documentation for :meth:`sge.Game.event_key_press` for
         more information.
@@ -536,7 +548,7 @@ class Room(object):
         pass
 
     def event_key_release(self, key):
-        u"""Key release event.
+        """Key release event.
 
         See the documentation for :meth:`sge.Game.event_key_release` for
         more information.
@@ -545,7 +557,7 @@ class Room(object):
         pass
 
     def event_mouse_move(self, x, y):
-        u"""Mouse move event.
+        """Mouse move event.
 
         See the documentation for :meth:`sge.Game.event_mouse_move` for
         more information.
@@ -554,7 +566,7 @@ class Room(object):
         pass
 
     def event_mouse_button_press(self, button):
-        u"""Mouse button press event.
+        """Mouse button press event.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_press` for more information.
@@ -563,7 +575,7 @@ class Room(object):
         pass
 
     def event_mouse_button_release(self, button):
-        u"""Mouse button release event.
+        """Mouse button release event.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_release` for more
@@ -573,7 +585,7 @@ class Room(object):
         pass
 
     def event_joystick_axis_move(self, name, ID, axis, value):
-        u"""Joystick axis move event.
+        """Joystick axis move event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_axis_move` for more information.
@@ -582,7 +594,7 @@ class Room(object):
         pass
 
     def event_joystick_hat_move(self, name, ID, hat, x, y):
-        u"""Joystick HAT move event.
+        """Joystick HAT move event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_hat_move` for more information.
@@ -591,7 +603,7 @@ class Room(object):
         pass
 
     def event_joystick_trackball_move(self, name, ID, ball, x, y):
-        u"""Joystick trackball move event.
+        """Joystick trackball move event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_trackball_move` for more
@@ -601,7 +613,7 @@ class Room(object):
         pass
 
     def event_joystick_button_press(self, name, ID, button):
-        u"""Joystick button press event.
+        """Joystick button press event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_press` for more
@@ -611,7 +623,7 @@ class Room(object):
         pass
 
     def event_joystick_button_release(self, name, ID, button):
-        u"""Joystick button release event.
+        """Joystick button release event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_release` for more
@@ -621,7 +633,7 @@ class Room(object):
         pass
 
     def event_gain_keyboard_focus(self):
-        u"""Gain keyboard focus event.
+        """Gain keyboard focus event.
 
         See the documentation for
         :meth:`sge.Game.event_gain_keyboard_focus` for more information.
@@ -630,19 +642,19 @@ class Room(object):
         pass
 
     def event_lose_keyboard_focus(self):
-        u"""Lose keyboard focus event.
+        """Lose keyboard focus event.
 
         See the documentation for
         :meth:`sge.Game.event_lose_keyboard_focus` for more information.
 
         """
         if sge.DEBUG:
-            bad_name = u"event_loose_keyboard_focus"
+            bad_name = "event_loose_keyboard_focus"
             if hasattr(self, bad_name):
                 sge._scold_user_on_lose_vs_loose(bad_name)
 
     def event_gain_mouse_focus(self):
-        u"""Gain mouse focus event.
+        """Gain mouse focus event.
 
         See the documentation for
         :meth:`sge.Game.event_gain_mouse_focus` for more information.
@@ -651,19 +663,19 @@ class Room(object):
         pass
 
     def event_lose_mouse_focus(self):
-        u"""Lose mouse focus event.
+        """Lose mouse focus event.
 
         See the documentation for
         :meth:`sge.Game.event_lose_mouse_focus` for more information.
 
         """
         if sge.DEBUG:
-            bad_name = u"event_loose_mouse_focus"
+            bad_name = "event_loose_mouse_focus"
             if hasattr(self, bad_name):
                 sge._scold_user_on_lose_vs_loose(bad_name)
 
     def event_close(self):
-        u"""Close event.
+        """Close event.
 
         See the documentation for :meth:`sge.Game.event_close` for more
         information.  This is always called before any game close events
@@ -673,7 +685,7 @@ class Room(object):
         pass
 
     def event_paused_key_press(self, key, char):
-        u"""Key press event when paused.
+        """Key press event when paused.
 
         See the documentation for :meth:`sge.Game.event_key_press` for
         more information.
@@ -682,7 +694,7 @@ class Room(object):
         pass
 
     def event_paused_key_release(self, key):
-        u"""Key release event when paused.
+        """Key release event when paused.
 
         See the documentation for :meth:`sge.Game.event_key_release` for
         more information.
@@ -691,7 +703,7 @@ class Room(object):
         pass
 
     def event_paused_mouse_move(self, x, y):
-        u"""Mouse move event when paused.
+        """Mouse move event when paused.
 
         See the documentation for :meth:`sge.Game.event_mouse_move` for
         more information.
@@ -700,7 +712,7 @@ class Room(object):
         pass
 
     def event_paused_mouse_button_press(self, button):
-        u"""Mouse button press event when paused.
+        """Mouse button press event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_press` for more information.
@@ -709,7 +721,7 @@ class Room(object):
         pass
 
     def event_paused_mouse_button_release(self, button):
-        u"""Mouse button release event when paused.
+        """Mouse button release event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_release` for more
@@ -719,7 +731,7 @@ class Room(object):
         pass
 
     def event_paused_joystick_axis_move(self, name, ID, axis, value):
-        u"""Joystick axis move event when paused.
+        """Joystick axis move event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_axis_move` for more information.
@@ -728,7 +740,7 @@ class Room(object):
         pass
 
     def event_paused_joystick_hat_move(self, name, ID, hat, x, y):
-        u"""Joystick HAT move event when paused.
+        """Joystick HAT move event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_hat_move` for more information.
@@ -737,7 +749,7 @@ class Room(object):
         pass
 
     def event_paused_joystick_trackball_move(self, name, ID, ball, x, y):
-        u"""Joystick trackball move event when paused.
+        """Joystick trackball move event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_trackball_move` for more
@@ -747,7 +759,7 @@ class Room(object):
         pass
 
     def event_paused_joystick_button_press(self, name, ID, button):
-        u"""Joystick button press event when paused.
+        """Joystick button press event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_press` for more
@@ -757,7 +769,7 @@ class Room(object):
         pass
 
     def event_paused_joystick_button_release(self, name, ID, button):
-        u"""Joystick button release event when paused.
+        """Joystick button release event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_release` for more
@@ -767,7 +779,7 @@ class Room(object):
         pass
 
     def event_paused_gain_keyboard_focus(self):
-        u"""Gain keyboard focus event when paused.
+        """Gain keyboard focus event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_gain_keyboard_focus` for more information.
@@ -776,19 +788,19 @@ class Room(object):
         pass
 
     def event_paused_lose_keyboard_focus(self):
-        u"""Lose keyboard focus event when paused.
+        """Lose keyboard focus event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_lose_keyboard_focus` for more information.
 
         """
         if sge.DEBUG:
-            bad_name = u"event_paused_loose_keyboard_focus"
+            bad_name = "event_paused_loose_keyboard_focus"
             if hasattr(self, bad_name):
                 sge._scold_user_on_lose_vs_loose(bad_name)
 
     def event_paused_gain_mouse_focus(self):
-        u"""Gain mouse focus event when paused.
+        """Gain mouse focus event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_gain_mouse_focus` for more information.
@@ -797,19 +809,19 @@ class Room(object):
         pass
 
     def event_paused_lose_mouse_focus(self):
-        u"""Lose mouse focus event when paused.
+        """Lose mouse focus event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_lose_mouse_focus` for more information.
 
         """
         if sge.DEBUG:
-            bad_name = u"event_paused_loose_mouse_focus"
+            bad_name = "event_paused_loose_mouse_focus"
             if hasattr(self, bad_name):
                 sge._scold_user_on_lose_vs_loose(bad_name)
 
     def event_paused_close(self):
-        u"""Close event when paused.
+        """Close event when paused.
 
         See the documentation for :meth:`sge.Game.event_close` for more
         information.
@@ -863,21 +875,3 @@ class Room(object):
             obj.bbox_height = self._object_start_bbox_height[obj.id]
             obj.collision_ellipse = self._object_start_collision_ellipse[obj.id]
             obj.collision_precise = self._object_start_collision_precise[obj.id]
-
-
-class _Projection(sge.StellarClass):
-
-    # Object which destroys itself after being shown for one frame.
-
-    def event_create(self):
-        self.detects_collisions = False
-        self.death_alarm = 2
-
-    def event_step(self, time_passed):
-        self.death_alarm -= 1
-        if self.death_alarm <= 0:
-            self.destroy()
-
-    def event_destroy(self):
-        if self.sprite is not None:
-            del sge.game.sprites[self.sprite.id]
