@@ -1,4 +1,4 @@
-# Copyright (C) 2012, 2013 Julian Marchant <onpon4@riseup.net>
+# Copyright (C) 2012, 2013, 2014 Julian Marchant <onpon4@riseup.net>
 # 
 # This file is part of the Pygame SGE.
 # 
@@ -15,11 +15,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with the Pygame SGE.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import division
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import os
 
 import pygame
@@ -27,12 +22,12 @@ import pygame
 import sge
 
 
-__all__ = [u'Sound']
+__all__ = ['Sound']
 
 
-class Sound(object):
+class Sound:
 
-    u"""Sound handling class.
+    """Sound handling class.
 
     This class stores and plays sound effects.  Note that this is
     inefficient for large music files; for those, use :class:`sge.Music`
@@ -71,6 +66,10 @@ class Sound(object):
 
     """
 
+    # Defaults, to prevent problems with __del__
+    _sound = None
+    _channels = []
+
     @property
     def max_play(self):
         return len(self._channels)
@@ -102,7 +101,7 @@ class Sound(object):
             return 0
 
     def __init__(self, fname, ID=None, volume=100, max_play=1):
-        u"""Constructor method.
+        """Constructor method.
 
         Arguments:
 
@@ -132,10 +131,10 @@ class Sound(object):
                     pass
 
             if self._sound is None:
-                print(u"Directories searched:")
+                print("Directories searched:")
                 for d in sge.music_directories:
                     print(os.path.normpath(os.path.abspath(d)))
-                msg = u'File "{0}" not found.'.format(self.fname)
+                msg = 'File "{}" not found.'.format(self.fname)
                 raise IOError(msg)
 
             if ID is not None:
@@ -143,7 +142,7 @@ class Sound(object):
             else:
                 self.id = os.path.splitext(os.path.basename(fname))[0]
                 while self.id in sge.game.sounds:
-                    self.id += u"_"
+                    self.id += "_"
         else:
             self._sound = None
 
@@ -166,7 +165,7 @@ class Sound(object):
 
     def play(self, loops=1, volume=100, balance=0, maxtime=None,
              fade_time=None):
-        u"""Play the sound.
+        """Play the sound.
 
         Arguments:
 
@@ -227,7 +226,7 @@ class Sound(object):
                 sge.game._release_channel(self._temp_channels.pop(0))
 
     def stop(self, fade_time=None):
-        u"""Stop the sound.
+        """Stop the sound.
 
         Arguments:
 
@@ -240,17 +239,26 @@ class Sound(object):
             self._sound.stop()
 
     def pause(self):
-        u"""Pause playback of the sound."""
+        """Pause playback of the sound."""
         for channel in self._channels:
             channel.pause()
 
     def unpause(self):
-        u"""Resume playback of the sound if paused."""
+        """Resume playback of the sound if paused."""
         for channel in self._channels:
             channel.unpause()
 
+    def destroy(self):
+        """Destroy the sound."""
+        self.stop()
+
+        while self._channels:
+            sge.game._release_channel(self._channels.pop(0))
+
+        del sge.game.sounds[self.id]
+
     @staticmethod
     def stop_all():
-        u"""Stop playback of all sounds."""
+        """Stop playback of all sounds."""
         for i in sge.game.sounds:
             sge.game.sounds[i].stop()    
