@@ -15,11 +15,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with the Pygame SGE.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import division
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import os
 import warnings
 
@@ -28,12 +23,12 @@ import pygame
 import sge
 
 
-__all__ = [u'Sprite']
+__all__ = ['Sprite']
 
 
-class Sprite(object):
+class Sprite:
 
-    u"""Class which holds information for images and animations.
+    """Class which holds information for images and animations.
 
     This class stores images and information about how the SGE is to use
     those images.
@@ -125,6 +120,10 @@ class Sprite(object):
 
        The unique identifier of the sprite.  (Read-only)
 
+    .. attribute:: frames
+
+       The number of animation frames in the sprite.  (Read-only)
+
     """
 
     @property
@@ -189,11 +188,15 @@ class Sprite(object):
         else:
             self._bbox_y = -self.origin_y
 
+    @property
+    def frames(self):
+        return len(self._baseimages)
+
     def __init__(self, name=None, ID=None, width=None, height=None,
                  origin_x=0, origin_y=0, transparent=True, fps=60,
                  bbox_x=None, bbox_y=None, bbox_width=None,
                  bbox_height=None):
-        u"""Constructor method.
+        """Constructor method.
 
         Arguments:
 
@@ -234,7 +237,7 @@ class Sprite(object):
 
         """
         if sge.DEBUG:
-            print(u'Creating sprite "{0}"'.format(name))
+            print('Creating sprite "{}"'.format(name))
 
         self.name = name
         self._transparent = None
@@ -255,7 +258,7 @@ class Sprite(object):
                 i = 0
                 while self.id in sge.game.sprites:
                     i += 1
-                    self.id = u"{0}{1}".format(self.name, i)
+                    self.id = "{}{}".format(self.name, i)
 
             for path in sge.image_directories:
                 if os.path.isdir(path):
@@ -265,11 +268,11 @@ class Sprite(object):
                         if fname.startswith(name) and os.path.isfile(full_fname):
                             root, ext = os.path.splitext(fname)
                             if root == name:
-                                split = [name, u'']
-                            elif root.rsplit(u'-', 1)[0] == name:
-                                split = root.rsplit(u'-', 1)
-                            elif root.rsplit(u'_', 1)[0] == name:
-                                split = root.rsplit(u'_', 1)
+                                split = [name, '']
+                            elif root.rsplit('-', 1)[0] == name:
+                                split = root.rsplit('-', 1)
+                            elif root.rsplit('_', 1)[0] == name:
+                                split = root.rsplit('_', 1)
                             else:
                                 continue
 
@@ -280,7 +283,7 @@ class Sprite(object):
                                 while len(fname_frames) - 1 < n:
                                     fname_frames.append(None)
                                 fname_frames[n] = full_fname
-                            elif (split[1].startswith(u'strip') and
+                            elif (split[1].startswith('strip') and
                                   split[1][5:].isdigit()):
                                 fname_strip = full_fname
 
@@ -291,7 +294,7 @@ class Sprite(object):
                     self._baseimages.append(img)
                 except pygame.error:
                     if sge.DEBUG:
-                        print(u"Ignored {0}; not a valid image.".format(fname_single))
+                        print("Ignored {}; not a valid image.".format(fname_single))
 
             if not self._baseimages and any(fname_frames):
                 # Load the multiple images
@@ -301,18 +304,18 @@ class Sprite(object):
                             self._baseimages.append(pygame.image.load(fname))
                         except pygame.error:
                             if sge.DEBUG:
-                                print(u"Ignored {0}; not a valid image.".format(fname))
+                                print("Ignored {}; not a valid image.".format(fname))
 
             if not self._baseimages and fname_strip:
                 # Load the strip (sprite sheet)
                 root, ext = os.path.splitext(os.path.basename(fname_strip))
-                assert u'-' in root or u'_' in root
-                assert (root.rsplit(u'-', 1)[0] == name or
-                        root.rsplit(u'_', 1)[0] == name)
-                if root.rsplit(u'-', 1)[0] == name:
-                    split = root.rsplit(u'-', 1)
+                assert '-' in root or '_' in root
+                assert (root.rsplit('-', 1)[0] == name or
+                        root.rsplit('_', 1)[0] == name)
+                if root.rsplit('-', 1)[0] == name:
+                    split = root.rsplit('-', 1)
                 else:
-                    split = root.rsplit(u'_', 1)
+                    split = root.rsplit('_', 1)
 
                 try:
                     sheet = pygame.image.load(fname_strip)
@@ -321,19 +324,19 @@ class Sprite(object):
 
                     img_w = max(1, sheet.get_width()) // n
                     img_h = max(1, sheet.get_height())
-                    for x in xrange(0, img_w * n, img_w):
+                    for x in range(0, img_w * n, img_w):
                         rect = pygame.Rect(x, 0, img_w, img_h)
                         img = sheet.subsurface(rect)
                         self._baseimages.append(img)
                 except pygame.error:
                     if sge.DEBUG:
-                        print(u"Ignored {0}; not a valid image.".format(fname_strip))
+                        print("Ignored {}; not a valid image.".format(fname_strip))
 
             if not self._baseimages:
-                print(u"Directories searched:")
+                print("Directories searched:")
                 for d in sge.image_directories:
                     print(os.path.normpath(os.path.abspath(d)))
-                msg = u'Files for sprite name "{0}" not found.'.format(name)
+                msg = 'Files for sprite name "{}" not found.'.format(name)
                 raise IOError(msg)
         else:
             # Name is None; default to a blank rectangle.
@@ -342,8 +345,11 @@ class Sprite(object):
             if height is None:
                 height = 32
 
+            self._w = width
+            self._h = height
+
             # Choose name
-            self.name = u"sge-pygame-dynamicsprite"
+            self.name = "sge-pygame-dynamicsprite"
             if ID is not None:
                 self.id = ID
             else:
@@ -351,11 +357,9 @@ class Sprite(object):
                 while self.id in sge.game.sprites:
                     self.id += 1
 
-            img = pygame.Surface((width, height), pygame.SRCALPHA)
-            img.fill(pygame.Color(0, 0, 0, 0))
-            self._baseimages.append(img)
+            self.append_frame()
             if sge.DEBUG:
-                print(u"renamed to {0}, ID is {1}".format(self.name, self.id))
+                print("renamed to {}, ID is {}".format(self.name, self.id))
 
         if width is None:
             width = 1
@@ -387,8 +391,38 @@ class Sprite(object):
         self._refresh()
         sge.game.sprites[self.id] = self
 
+    def append_frame(self):
+        """Append a new blank frame to the end of the sprite."""
+        img = pygame.Surface((self.width, self.height), pygame.SRCALPHA)
+        img.fill(pygame.Color(0, 0, 0, 0))
+        self._baseimages.append(img)
+
+    def insert_frame(self, frame):
+        """Insert a new blank frame into the sprite.
+
+        Arguments:
+
+        - ``frame`` -- The frame of the sprite to insert the new frame
+          in front of, where ``0`` is the first frame.
+
+        """
+        img = pygame.Surface((self.width, self.height), pygame.SRCALPHA)
+        img.fill(pygame.Color(0, 0, 0, 0))
+        self._baseimages.insert(frame, img)
+
+    def delete_frame(self, frame):
+        """Delete a frame from the sprite.
+
+        Arguments:
+
+        - ``frame`` -- The frame of the sprite to delete, where ``0`` is
+          the first frame.
+
+        """
+        del self._baseimages[frame]
+
     def draw_dot(self, x, y, color, frame=None):
-        u"""Draw a single-pixel dot on the sprite.
+        """Draw a single-pixel dot on the sprite.
 
         Arguments:
 
@@ -403,15 +437,15 @@ class Sprite(object):
 
         """
         color = sge._get_pygame_color(color)
-        for i in xrange(len(self._baseimages)):
-            if frame is None or frame % len(self._baseimages) == i:
+        for i in range(self.frames):
+            if frame is None or frame % self.frames == i:
                 self._baseimages[i].set_at((x, y), color)
 
         self._refresh()
 
     def draw_line(self, x1, y1, x2, y2, color, thickness=1, anti_alias=False,
                   frame=None):
-        u"""Draw a line segment on the sprite.
+        """Draw a line segment on the sprite.
 
         Arguments:
 
@@ -434,8 +468,8 @@ class Sprite(object):
         color = sge._get_pygame_color(color)
         thickness = abs(thickness)
 
-        for i in xrange(len(self._baseimages)):
-            if frame is None or frame % len(self._baseimages) == i:
+        for i in range(self.frames):
+            if frame is None or frame % self.frames == i:
                 if anti_alias and thickness == 1:
                     pygame.draw.aaline(self._baseimages[i], color, (x1, y1),
                                        (x2, y2))
@@ -447,7 +481,7 @@ class Sprite(object):
 
     def draw_rectangle(self, x, y, width, height, fill=None, outline=None,
                        outline_thickness=1, frame=None):
-        u"""Draw a rectangle on the sprite.
+        """Draw a rectangle on the sprite.
 
         Arguments:
 
@@ -476,8 +510,8 @@ class Sprite(object):
 
         rect = pygame.Rect(x, y, width, height)
 
-        for i in xrange(len(self._baseimages)):
-            if frame is None or frame % len(self._baseimages) == i:
+        for i in range(self.frames):
+            if frame is None or frame % self.frames == i:
                 if fill is not None:
                     pygame.draw.rect(self._baseimages[i],
                                      sge._get_pygame_color(fill), rect, 0)
@@ -491,7 +525,7 @@ class Sprite(object):
 
     def draw_ellipse(self, x, y, width, height, fill=None, outline=None,
                      outline_thickness=1, anti_alias=False, frame=None):
-        u"""Draw an ellipse on the sprite.
+        """Draw an ellipse on the sprite.
 
         Arguments:
 
@@ -521,8 +555,8 @@ class Sprite(object):
 
         rect = pygame.Rect(x, y, width, height)
 
-        for i in xrange(len(self._baseimages)):
-            if frame is None or frame % len(self._baseimages) == i:
+        for i in range(self.frames):
+            if frame is None or frame % self.frames == i:
                 if fill is not None:
                     c = sge._get_pygame_color(fill)
                     pygame.draw.ellipse(self._baseimages[i], c, rect)
@@ -536,7 +570,7 @@ class Sprite(object):
 
     def draw_circle(self, x, y, radius, fill=None, outline=None,
                     outline_thickness=1, anti_alias=False, frame=None):
-        u"""Draw a circle on the sprite.
+        """Draw a circle on the sprite.
 
         Arguments:
 
@@ -563,8 +597,8 @@ class Sprite(object):
             # There's no point in trying in this case.
             return
 
-        for i in xrange(len(self._baseimages)):
-            if frame is None or frame % len(self._baseimages) == i:
+        for i in range(self.frames):
+            if frame is None or frame % self.frames == i:
                 if fill is not None:
                     c = sge._get_pygame_color(fill)
                     pygame.draw.circle(self._baseimages[i], c, (x, y), radius)
@@ -577,7 +611,7 @@ class Sprite(object):
         self._refresh()
 
     def draw_sprite(self, sprite, image, x, y, frame=None, blend_mode=None):
-        u"""Draw another sprite on the sprite.
+        """Draw another sprite on the sprite.
 
         Arguments:
 
@@ -618,8 +652,8 @@ class Sprite(object):
         y -= sprite.origin_y
         image %= len(sprite._baseimages)
 
-        if blend_mode & sge.BLEND_SCREEN:
-            w = u"Screen blend mode not supported. Normal blending used instead."
+        if blend_mode is not None and blend_mode & sge.BLEND_SCREEN:
+            w = "Screen blend mode not supported. Normal blending used instead."
             warnings.warn(w)
 
         pygame_flags = {
@@ -635,17 +669,17 @@ class Sprite(object):
             sge.BLEND_RGB_MAXIMUM: pygame.BLEND_RGB_MAX
             }.setdefault(blend_mode, 0)
 
-        for i in xrange(len(self._baseimages)):
-            if frame is None or frame % len(self._baseimages) == i:
-                self._baseimages[i].blit(sprite._baseimages[i], (x, y),
+        for i in range(self.frames):
+            if frame is None or frame % self.frames == i:
+                self._baseimages[i].blit(sprite._baseimages[image], (x, y),
                                          None, pygame_flags)
 
         self._refresh()
 
     def draw_text(self, font, text, x, y, width=None, height=None,
-                  color=u"black", halign=sge.ALIGN_LEFT, valign=sge.ALIGN_TOP,
+                  color="black", halign=sge.ALIGN_LEFT, valign=sge.ALIGN_TOP,
                   anti_alias=True, frame=None):
-        u"""Draw text on the sprite.
+        """Draw text on the sprite.
 
         Arguments:
 
@@ -718,7 +752,7 @@ class Sprite(object):
         text_rect = text_surf.get_rect()
         box_rect = box_surf.get_rect()
 
-        for i in xrange(len(lines)):
+        for i in range(len(lines)):
             rendered_text = font._font.render(lines[i], anti_alias, color)
             rect = rendered_text.get_rect()
             rect.top = i * font._font.get_linesize()
@@ -759,14 +793,14 @@ class Sprite(object):
         else:
             box_rect.top = y
 
-        for i in xrange(len(self._baseimages)):
-            if frame is None or frame % len(self._baseimages) == i:
+        for i in range(self.frames):
+            if frame is None or frame % self.frames == i:
                 self._baseimages[i].blit(box_surf, box_rect)
 
         self._refresh()
 
     def draw_clear(self, frame=None):
-        u"""Erase everything from the sprite.
+        """Erase everything from the sprite.
 
         Arguments:
 
@@ -774,8 +808,8 @@ class Sprite(object):
           the first frame; set to :const:`None` to clear all frames.
 
         """
-        for i in xrange(len(self._baseimages)):
-            if frame is None or frame % len(self._baseimages) == i:
+        for i in range(self.frames):
+            if frame is None or frame % self.frames == i:
                 if self._baseimages[i].get_flags() & pygame.SRCALPHA:
                     color = pygame.Color(0, 0, 0, 0)
                 else:
@@ -786,7 +820,7 @@ class Sprite(object):
         self._refresh()
 
     def save(self, fname):
-        u"""Save the sprite to an image file.
+        """Save the sprite to an image file.
 
         Arguments:
 
@@ -801,24 +835,90 @@ class Sprite(object):
         """
         # Assuming self.width and self.height are the size of all
         # surfaces in _baseimages (this should be the case).
-        w = self.width * len(self._baseimages)
+        w = self.width * self.frames
         h = self.height
         reel = pygame.Surface((w, h), pygame.SRCALPHA)
         reel.fill(pygame.Color(0, 0, 0, 0))
 
-        for i in xrange(len(self._baseimages)):
+        for i in range(self.frames):
             reel.blit(self._baseimages[i], (self.width * i, 0))
 
         try:
             pygame.image.save(reel, fname)
         except pygame.error:
-            m = u'Couldn\'t save to "{0}"'.format(
+            m = 'Couldn\'t save to "{}"'.format(
                 os.path.normpath(os.path.realpath(fname)))
             raise IOError(m)
 
+    def destroy(self):
+        """Destroy the sprite.
+
+        .. note::
+
+           If the sprite is being used, for example, by a
+           :class:`sge.StellarClass` object, it will not be completely
+           destroyed until this use stops.
+
+        """
+        del sge.game.sprites[self.id]
+
+    @classmethod
+    def from_tileset(cls, name, ID=None, x=0, y=0, columns=1, rows=1, xsep=0,
+                     ysep=0, width=1, height=1, origin_x=0, origin_y=0,
+                     transparent=True, fps=0, bbox_x=None, bbox_y=None,
+                     bbox_width=None, bbox_height=None):
+        """Return a sprite based on the tiles in a tileset.
+
+        Arguments:
+
+        - ``name`` -- The base name of the image file containing the
+          tileset, not including the file extension, to be found in one
+          of the paths specified in :data:`sge.image_directories`.
+        - ``x`` -- The horizontal location relative to the image of the
+          top-leftmost tile in the tileset.
+        - ``y`` -- The vertical location relative to the image of the
+          top-leftmost tile in the tileset.
+        - ``columns`` -- The number of columns in the tileset.
+        - ``rows`` -- The number of rows in the tileset.
+        - ``xsep`` -- The spacing between columns in the tileset.
+        - ``ysep`` -- The spacing between rows in the tileset.
+        - ``width`` -- The width of the tiles.
+        - ``height`` -- The height of the tiles.
+
+        For all other arguments, see the documentation for
+        :meth:`Sprite.__init__`.
+
+        Each tile in the tileset becomes a subimage of the returned
+        sprite, ordered first from left to right and then from top to
+        bottom.
+
+        """
+        self = cls(name, ID=ID, width=width, height=height, origin_x=origin_x,
+                   origin_y=origin_y, transparent=transparent, fps=fps,
+                   bbox_x=bbox_x, bbox_y=bbox_y, bbox_width=bbox_width,
+                   bbox_height=bbox_height)
+        tileset = Sprite(name, origin_x=x, origin_y=y)
+
+        for i in range(1, rows):
+            for j in range(1, columns):
+                self.append_frame()
+
+        self.draw_clear()
+
+        for i in range(rows):
+            for j in range(columns):
+                frame = i * columns + j
+                x_ = (width + xsep) * j
+                y_ = (height + ysep) * i
+                self.draw_sprite(tileset, 0, -x_ - self.origin_x,
+                                 -y_ - self.origin_y, frame=frame)
+
+        tileset.destroy()
+        return self
+
     @classmethod
     def from_screenshot(cls, x=0, y=0, width=None, height=None):
-        u"""Return the current display on the screen as a sprite.
+        """Return the current display on the screen as a sprite.
 
         Arguments:
 
@@ -872,7 +972,7 @@ class Sprite(object):
         # methods to work properly, specifically whenever ``width`` and
         # ``height`` are set.  As a result, setting ``width`` and
         # ``height`` is destructive in this implementation.
-        for i in xrange(len(self._baseimages)):
+        for i in range(self.frames):
             if sge.game.scale_smooth:
                 try:
                     self._baseimages[i] = pygame.transform.smoothscale(
@@ -911,8 +1011,10 @@ class Sprite(object):
                    blend=None):
         num %= len(self._images)
         # Return the properly sized surface.
-        if (xscale, yscale, rotation, alpha, blend) in self._images[num]:
-            return self._images[num][(xscale, yscale, rotation, alpha, blend)]
+        t = (sge.game._xscale, sge.game._yscale, xscale, yscale, rotation,
+             alpha, blend)
+        if t in self._images[num]:
+            return self._images[num][t]
         else:
             # Hasn't been scaled to this size yet
             if xscale != 0 and yscale != 0:
@@ -941,30 +1043,40 @@ class Sprite(object):
                 img = pygame.Surface((1, 1))
                 img.set_colorkey((0, 0, 0), pygame.RLEACCEL)
 
-            self._images[num][(xscale, yscale, rotation, alpha, blend)] = img
+            self._images[num][t] = img
             return img
 
-    def _get_precise_mask(self, num):
+    def _get_precise_mask(self, num, xscale, yscale, rotation):
         # Return a precise mask (2D list of True/False values) for the
         # given image index.
-        if num in self._masks:
-            return self._masks[num]
+        mask_id = (num, xscale, yscale, rotation)
+        if mask_id in self._masks:
+            return self._masks[mask_id]
         else:
-            image = self._get_image(num)
+            #image = self._get_image(num, xscale, yscale, rotation)
+            image = self._set_transparency(self._baseimages[num])
+            xflip = xscale < 0
+            yflip = yscale < 0
+            image = pygame.transform.flip(image, xflip, yflip)
+            image = pygame.transform.scale(image, (self.width * abs(xscale),
+                                                   self.height * abs(yscale)))
+            if rotation:
+                image = pygame.transform.rotate(image, rotation)
+
             image.lock()
             mask = []
             if image.get_flags() & pygame.SRCALPHA:
-                for x in xrange(image.get_width()):
+                for x in range(image.get_width()):
                     mask.append([])
-                    for y in xrange(image.get_height()):
+                    for y in range(image.get_height()):
                         mask[x].append(image.get_at((x, y)).a > 0)
             else:
                 colorkey = image.get_colorkey()
-                for x in xrange(image.get_width()):
+                for x in range(image.get_width()):
                     mask.append([])
-                    for y in xrange(image.get_height()):
+                    for y in range(image.get_height()):
                         mask[x].append(image.get_at((x, y)) == colorkey)
 
             image.unlock()
-            self._masks[num] = mask
+            self._masks[mask_id] = mask
             return mask
