@@ -31,12 +31,12 @@ import pygame
 import sge
 
 
-__all__ = [u'StellarClass', u'Mouse']
+__all__ = ['StellarClass', 'Mouse', '_PygameProjectionSprite']
 
 
 class StellarClass(object):
 
-    u"""Class for game objects.
+    """Class for game objects.
 
     This class is used for game objects, such as the player, enemies,
     bullets, and the HUD.  Generally, each type of object has its own
@@ -282,9 +282,9 @@ class StellarClass(object):
         self._pygame_sprite.rect = pygame.Rect(0, 0, 1, 1)
 
         # Make the mouse have a higher Z-axis value
-        if (u"mouse" in sge.game.objects and
-                sge.game.objects[u"mouse"].z <= value):
-            sge.game.objects[u"mouse"]._z = value + 1
+        if ("mouse" in sge.game.objects and
+                sge.game.objects["mouse"].z <= value):
+            sge.game.objects["mouse"]._z = value + 1
 
     @property
     def sprite(self):
@@ -327,7 +327,6 @@ class StellarClass(object):
     def collision_ellipse(self, value):
         if value != self._collision_ellipse:
             self._collision_ellipse = value
-            self._set_mask()
 
     @property
     def collision_precise(self):
@@ -337,7 +336,6 @@ class StellarClass(object):
     def collision_precise(self, value):
         if value != self._collision_precise:
             self._collision_precise = value
-            self._set_mask()
 
     @property
     def bbox_left(self):
@@ -461,7 +459,7 @@ class StellarClass(object):
                  xvelocity=0, yvelocity=0, image_index=0, image_fps=None,
                  image_xscale=1, image_yscale=1, image_rotation=0,
                  image_alpha=255, image_blend=None):
-        u"""Constructor method.
+        """Constructor method.
 
         Arguments:
 
@@ -503,6 +501,7 @@ class StellarClass(object):
                             sprite_bbox_height)
         self._collision_ellipse = collision_ellipse
         self._collision_precise = collision_precise
+        self._collision_areas = []
         self._colliders = []
 
         if ID is not None:
@@ -536,10 +535,11 @@ class StellarClass(object):
 
         self._rect = pygame.Rect(self.bbox_x, self.bbox_y, self.bbox_width,
                                  self.bbox_height)
-        if self.id != u"mouse":
+        if self.id != "mouse":
             self._pygame_sprite = _PygameSprite(self)
         else:
             self._pygame_sprite = _FakePygameSprite(self)
+
         self._set_mask()
 
         self.z = z
@@ -558,7 +558,7 @@ class StellarClass(object):
         self._start_collision_precise = self.collision_precise
 
     def collides(self, other, x=None, y=None):
-        u"""Return whether or not this object collides with another.
+        """Return whether or not this object collides with another.
 
         Arguments:
 
@@ -601,43 +601,49 @@ class StellarClass(object):
             if (self.collision_precise or self.collision_ellipse or
                     other.collision_precise or other.collision_ellipse):
                 # Use masks.
-                self_rect = pygame.Rect(round(self.bbox_left + x),
-                                        round(self.bbox_top + y),
-                                        self.bbox_width, self.bbox_height)
-                other_rect = pygame.Rect(round(other.bbox_left),
-                                         round(other.bbox_top),
-                                         other.bbox_width, other.bbox_height)
+                self._set_mask()
+                other._set_mask()
 
-                if self.sprite is not None:
-                    # Offset for rotation
-                    w, h = self.sprite._get_image(
-                        self.image_index, self.image_xscale,
-                        self.image_yscale, self.image_rotation).get_size()
+                if self._hitmask and other._hitmask:
+                    self_rect = pygame.Rect(round(self.bbox_left + x),
+                                            round(self.bbox_top + y),
+                                            len(self._hitmask),
+                                            len(self._hitmask[0]))
+                    other_rect = pygame.Rect(round(other.bbox_left),
+                                             round(other.bbox_top),
+                                             len(other._hitmask),
+                                             len(other._hitmask[0]))
 
-                    nw, nh = self.sprite._get_image(
-                        self.image_index, self.image_xscale,
-                        self.image_yscale).get_size()
+                    if self.sprite is not None:
+                        # Offset for rotation
+                        w, h = self.sprite._get_image(
+                            self.image_index, self.image_xscale,
+                            self.image_yscale, self.image_rotation).get_size()
 
-                    offset = _get_rotation_offset(
-                        self.sprite.origin_x, self.sprite.origin_y,
-                        self.image_rotation, w, h, nw, nh)
+                        nw, nh = self.sprite._get_image(
+                            self.image_index, self.image_xscale,
+                            self.image_yscale).get_size()
 
-                    self_rect.left -= offset[0]
-                    self_rect.top -= offset[1]
+                        offset = _get_rotation_offset(
+                            self.sprite.origin_x, self.sprite.origin_y,
+                            self.image_rotation, w, h, nw, nh)
 
-                collide_rect = self_rect.clip(other_rect)
-                self_xoffset = collide_rect.left - self_rect.left
-                self_yoffset = collide_rect.top - self_rect.top
-                other_xoffset = collide_rect.left - other_rect.left
-                other_yoffset = collide_rect.top - other_rect.top
+                        self_rect.left -= offset[0] // sge.game._xscale
+                        self_rect.top -= offset[1] // sge.game._yscale
 
-                for a in xrange(collide_rect.w):
-                    for b in xrange(collide_rect.h):
-                        if (self._hitmask[
-                                a + self_xoffset][b + self_yoffset] and
-                            other._hitmask[
-                                a + other_xoffset][b + other_yoffset]):
-                            return True
+                    collide_rect = self_rect.clip(other_rect)
+                    self_xoffset = collide_rect.left - self_rect.left
+                    self_yoffset = collide_rect.top - self_rect.top
+                    other_xoffset = collide_rect.left - other_rect.left
+                    other_yoffset = collide_rect.top - other_rect.top
+
+                    for a in xrange(collide_rect.w):
+                        for b in xrange(collide_rect.h):
+                            if (self._hitmask[
+                                    a + self_xoffset][b + self_yoffset] and
+                                other._hitmask[
+                                    a + other_xoffset][b + other_yoffset]):
+                                return True
                         
             else:
                 # Use bounding boxes.
@@ -650,7 +656,7 @@ class StellarClass(object):
         return False
 
     def set_alarm(self, alarm_id, value):
-        u"""Set an alarm.
+        """Set an alarm.
 
         Arguments:
 
@@ -671,7 +677,7 @@ class StellarClass(object):
             del self._alarms[alarm_id]
 
     def get_alarm(self, alarm_id):
-        u"""Return the value of an alarm.
+        """Return the value of an alarm.
 
         Arguments:
 
@@ -686,28 +692,23 @@ class StellarClass(object):
             return None
 
     def destroy(self):
-        u"""Destroy the object."""
+        """Destroy the object."""
+        sge.game._background_changed = True
         self.event_destroy()
         self._pygame_sprite.kill()
         del sge.game.objects[self.id]
 
         for room in sge.game.rooms:
-            new_objects = []
-            for obj in room.objects:
-                if obj is not self:
-                    new_objects.append(obj)
-            room.objects = new_objects
+            while self in room.objects:
+                room.objects.remove(self)
 
             for cls in room.objects_by_class:
                 if isinstance(self, cls):
-                    new_objects = []
-                    for obj in room.objects_by_class[cls]:
-                        if obj is not self:
-                            new_objects.append(obj)
-                    room.objects_by_class[cls] = new_objects
+                    while self in room.objects_by_class[cls]:
+                        room.objects_by_class[cls].remove(self)
 
     def event_create(self):
-        u"""Create event.
+        """Create event.
 
         Called when the object is created.  It is always called after
         any room start events occurring at the same time.
@@ -716,11 +717,11 @@ class StellarClass(object):
         pass
 
     def event_destroy(self):
-        u"""Destroy event."""
+        """Destroy event."""
         pass
 
     def event_begin_step(self, time_passed):
-        u"""Begin step event.
+        """Begin step event.
 
         This event is executed each frame before automatic updates to
         objects (such as the effects of the speed variables).
@@ -739,7 +740,7 @@ class StellarClass(object):
         pass
 
     def event_step(self, time_passed):
-        u"""Step event.
+        """Step event.
 
         This event is executed each frame after automatic updates to
         objects (such as the effects of the speed variables), but before
@@ -752,7 +753,7 @@ class StellarClass(object):
         pass
 
     def event_end_step(self, time_passed):
-        u"""Step event.
+        """Step event.
 
         This event is executed each frame after automatic updates to
         objects (such as the effects of the speed variables), but before
@@ -765,7 +766,7 @@ class StellarClass(object):
         pass
 
     def event_alarm(self, alarm_id):
-        u"""Alarm event.
+        """Alarm event.
 
         Called when the value of an alarm reaches 0.
 
@@ -778,7 +779,7 @@ class StellarClass(object):
         pass
 
     def event_animation_end(self):
-        u"""Animation End event.
+        """Animation End event.
 
         Called when an animation cycle ends.
 
@@ -786,7 +787,7 @@ class StellarClass(object):
         pass
 
     def event_key_press(self, key, char):
-        u"""Key press event.
+        """Key press event.
 
         See the documentation for :meth:`sge.Game.event_key_press` for
         more information.
@@ -795,7 +796,7 @@ class StellarClass(object):
         pass
 
     def event_key_release(self, key):
-        u"""Key release event.
+        """Key release event.
 
         See the documentation for :meth:`sge.Game.event_key_release` for
         more information.
@@ -804,7 +805,7 @@ class StellarClass(object):
         pass
 
     def event_mouse_move(self, x, y):
-        u"""Mouse move event.
+        """Mouse move event.
 
         See the documentation for :meth:`sge.Game.event_mouse_move` for
         more information.
@@ -813,7 +814,7 @@ class StellarClass(object):
         pass
 
     def event_mouse_button_press(self, button):
-        u"""Mouse button press event.
+        """Mouse button press event.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_press` for more information.
@@ -822,7 +823,7 @@ class StellarClass(object):
         pass
 
     def event_mouse_button_release(self, button):
-        u"""Mouse button release event.
+        """Mouse button release event.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_release` for more
@@ -832,7 +833,7 @@ class StellarClass(object):
         pass
 
     def event_joystick_axis_move(self, name, ID, axis, value):
-        u"""Joystick axis move event.
+        """Joystick axis move event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_axis_move` for more information.
@@ -841,7 +842,7 @@ class StellarClass(object):
         pass
 
     def event_joystick_hat_move(self, name, ID, hat, x, y):
-        u"""Joystick HAT move event.
+        """Joystick HAT move event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_hat_move` for more information.
@@ -850,7 +851,7 @@ class StellarClass(object):
         pass
 
     def event_joystick_trackball_move(self, name, ID, ball, x, y):
-        u"""Joystick trackball move event.
+        """Joystick trackball move event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_trackball_move` for more
@@ -860,7 +861,7 @@ class StellarClass(object):
         pass
 
     def event_joystick_button_press(self, name, ID, button):
-        u"""Joystick button press event.
+        """Joystick button press event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_press` for more
@@ -870,7 +871,7 @@ class StellarClass(object):
         pass
 
     def event_joystick_button_release(self, name, ID, button):
-        u"""Joystick button release event.
+        """Joystick button release event.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_release` for more
@@ -880,7 +881,7 @@ class StellarClass(object):
         pass
 
     def event_collision(self, other):
-        u"""Default collision event.
+        """Default collision event.
 
         Called when another object collides with this object and none of
         the directional collision events are appropriate.  In
@@ -896,7 +897,7 @@ class StellarClass(object):
         pass
 
     def event_collision_left(self, other):
-        u"""Left collision event.
+        """Left collision event.
 
         Called when another object collides with this object's left
         side.
@@ -912,7 +913,7 @@ class StellarClass(object):
         self.event_collision(other)
 
     def event_collision_right(self, other):
-        u"""Right collision event.
+        """Right collision event.
 
         Called when another object collides with this object's right
         side.
@@ -928,7 +929,7 @@ class StellarClass(object):
         self.event_collision(other)
 
     def event_collision_top(self, other):
-        u"""Top collision event.
+        """Top collision event.
 
         Called when another object collides with this object's top side.
 
@@ -943,7 +944,7 @@ class StellarClass(object):
         self.event_collision(other)
 
     def event_collision_bottom(self, other):
-        u"""Bottom collision event.
+        """Bottom collision event.
 
         Called when another object collides with this object's bottom
         side.
@@ -959,7 +960,7 @@ class StellarClass(object):
         self.event_collision(other)
 
     def event_inactive_step(self, time_passed):
-        u"""Step event when this object is inactive.
+        """Step event when this object is inactive.
 
         See the documentation for :meth:`sge.StellarClass.event_step`
         for more information.  The object is considered to be inactive
@@ -969,7 +970,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_end_step(self, time_passed):
-        u"""End step event when this object is inactive.
+        """End step event when this object is inactive.
 
         See the documentation for
         :meth:`sge.StellarClass.event_end_step` for more information.
@@ -980,7 +981,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_key_press(self, key, char):
-        u"""Key press event when this object is inactive.
+        """Key press event when this object is inactive.
 
         See the documentation for :meth:`sge.Game.event_key_press` for
         more information.  The object is considered to be inactive when
@@ -990,7 +991,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_key_release(self, key):
-        u"""Key release event when this object is inactive.
+        """Key release event when this object is inactive.
 
         See the documentation for :meth:`sge.Game.event_key_release` for
         more information.  The object is considered to be inactive when
@@ -1000,7 +1001,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_mouse_move(self, x, y):
-        u"""Mouse move event when this object is inactive.
+        """Mouse move event when this object is inactive.
 
         See the documentation for :meth:`sge.Game.event_mouse_move` for
         more information.  The object is considered to be inactive when
@@ -1010,7 +1011,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_mouse_button_press(self, button):
-        u"""Mouse button press event when this object is inactive.
+        """Mouse button press event when this object is inactive.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_press` for more information.
@@ -1021,7 +1022,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_mouse_button_release(self, button):
-        u"""Mouse button release event when this object is inactive.
+        """Mouse button release event when this object is inactive.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_release` for more
@@ -1032,7 +1033,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_joystick_axis_move(self, name, ID, axis, value):
-        u"""Joystick axis move event when this object is inactive.
+        """Joystick axis move event when this object is inactive.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_axis_move` for more information.
@@ -1043,7 +1044,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_joystick_hat_move(self, name, ID, hat, x, y):
-        u"""Joystick HAT move event when this object is inactive.
+        """Joystick HAT move event when this object is inactive.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_hat_move` for more information.
@@ -1054,7 +1055,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_joystick_trackball_move(self, name, ID, ball, x, y):
-        u"""Joystick trackball move event when this object is inactive.
+        """Joystick trackball move event when this object is inactive.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_trackball_move` for more
@@ -1065,7 +1066,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_joystick_button_press(self, name, ID, button):
-        u"""Joystick button press event when this object is inactive.
+        """Joystick button press event when this object is inactive.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_press` for more
@@ -1076,7 +1077,7 @@ class StellarClass(object):
         pass
 
     def event_inactive_joystick_button_release(self, name, ID, button):
-        u"""Joystick button release event when this object is inactive.
+        """Joystick button release event when this object is inactive.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_release` for more
@@ -1087,7 +1088,7 @@ class StellarClass(object):
         pass
 
     def event_paused_key_press(self, key, char):
-        u"""Key press event when paused.
+        """Key press event when paused.
 
         See the documentation for :meth:`sge.Game.event_key_press` for
         more information.
@@ -1096,7 +1097,7 @@ class StellarClass(object):
         pass
 
     def event_paused_key_release(self, key):
-        u"""Key release event when paused.
+        """Key release event when paused.
 
         See the documentation for :meth:`sge.Game.event_key_release` for
         more information.
@@ -1105,7 +1106,7 @@ class StellarClass(object):
         pass
 
     def event_paused_mouse_move(self, x, y):
-        u"""Mouse move event when paused.
+        """Mouse move event when paused.
 
         See the documentation for :meth:`sge.Game.event_mouse_move` for
         more information.
@@ -1114,7 +1115,7 @@ class StellarClass(object):
         pass
 
     def event_paused_mouse_button_press(self, button):
-        u"""Mouse button press event when paused.
+        """Mouse button press event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_press` for more information.
@@ -1123,7 +1124,7 @@ class StellarClass(object):
         pass
 
     def event_paused_mouse_button_release(self, button):
-        u"""Mouse button release event when paused.
+        """Mouse button release event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_mouse_button_release` for more
@@ -1133,7 +1134,7 @@ class StellarClass(object):
         pass
 
     def event_paused_joystick_axis_move(self, name, ID, axis, value):
-        u"""Joystick axis move event when paused.
+        """Joystick axis move event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_axis_move` for more information.
@@ -1142,7 +1143,7 @@ class StellarClass(object):
         pass
 
     def event_paused_joystick_hat_move(self, name, ID, hat, x, y):
-        u"""Joystick HAT move event when paused.
+        """Joystick HAT move event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_hat_move` for more information.
@@ -1151,7 +1152,7 @@ class StellarClass(object):
         pass
 
     def event_paused_joystick_trackball_move(self, name, ID, ball, x, y):
-        u"""Joystick trackball move event when paused.
+        """Joystick trackball move event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_trackball_move` for more
@@ -1161,7 +1162,7 @@ class StellarClass(object):
         pass
 
     def event_paused_joystick_button_press(self, name, ID, button):
-        u"""Joystick button press event when paused.
+        """Joystick button press event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_press` for more
@@ -1171,7 +1172,7 @@ class StellarClass(object):
         pass
 
     def event_paused_joystick_button_release(self, name, ID, button):
-        u"""Joystick button release event when paused.
+        """Joystick button release event when paused.
 
         See the documentation for
         :meth:`sge.Game.event_joystick_button_release` for more
@@ -1182,7 +1183,7 @@ class StellarClass(object):
 
     @classmethod
     def create(cls, *args, **kwargs):
-        u"""Create an object of this class in the current room.
+        """Create an object of this class in the current room.
 
         ``args`` and ``kwargs`` are passed to the constructor method of
         ``cls`` as arguments.  Calling
@@ -1213,11 +1214,48 @@ class StellarClass(object):
 
         # Movement
         if self.xvelocity or self.yvelocity:
-            if self.id != u"mouse":
+            if self.id != "mouse":
                 self.x += self.xvelocity * delta_mult
                 self.y += self.yvelocity * delta_mult
 
-        self._colliders = sge.game._colliders[:]
+        room = sge.game.current_room
+        area_size = room._collision_area_size
+        areas_x_start = int(self.bbox_left / area_size)
+        areas_x_num = int(math.ceil(self.bbox_width / area_size) + 1)
+        areas_y_start = int(self.bbox_top / area_size)
+        areas_y_num = int(math.ceil(self.bbox_height / area_size) + 1)
+
+        my_areas = []
+        for i in xrange(areas_x_start, areas_x_start + areas_x_num):
+            for j in xrange(areas_y_start, areas_y_start + areas_y_num):
+                if (i >= 0 and j >= 0 and room._collision_areas and
+                        i < len(room._collision_areas) and
+                        j < len(room._collision_areas[0])):
+                    my_areas.append((i, j))
+                elif None not in my_areas:
+                    my_areas.append(None)
+
+        for area in self._collision_areas:
+            if area not in my_areas:
+                if area is not None:
+                    i, j = area
+                    room_area = room._collision_areas[i][j]
+                else:
+                    room_area = room._collision_area_void
+
+                for i in xrange(len(room_area), 0, -1):
+                    if room_area[i - 1]() is self:
+                        del room_area[i - 1]
+
+        for area in my_areas:
+            if area not in self._collision_areas:
+                if area is not None:
+                    i, j = area
+                    room._collision_areas[i][j].append(weakref.ref(self))
+                else:
+                    room._collision_area_void.append(weakref.ref(self))
+
+        self._collision_areas = my_areas
 
     def _detect_collisions(self):
         for other_ref in self._colliders:
@@ -1279,16 +1317,9 @@ class StellarClass(object):
         # Properly set the hit mask based on the collision settings.
         if self.collision_precise:
             # Mask based on opacity of the current image.
-            left = self.bbox_x + self.sprite.origin_x
-            right = left + self.bbox_width
-            top = self.bbox_y + self.sprite.origin_y
-            bottom = top + self.bbox_height
-
-            mask = self.sprite._get_precise_mask(self.image_index)[left:right]
-            for i in xrange(len(mask)):
-                mask[i] = mask[i][top:bottom]
-
-            self._hitmask = mask
+            self._hitmask = self.sprite._get_precise_mask(
+                self.image_index, self.image_xscale, self.image_yscale,
+                self.image_rotation)
         elif self.collision_ellipse:
             # Elliptical mask based on bounding box.
             self._hitmask = [[False for y in xrange(self.bbox_height)]
@@ -1430,7 +1461,7 @@ class Mouse(StellarClass):
     @visible.setter
     def visible(self, value):
         if sge.DEBUG:
-            print(u"mouse.visible has been set to:", value)
+            print("mouse.visible has been set to:", value)
 
         self._visible = value
         self.set_cursor()
@@ -1442,7 +1473,7 @@ class Mouse(StellarClass):
         self.mouse_yprevious = self.mouse_y
         self.previous_speeds = []
 
-        super(Mouse, self).__init__(0, 0, 0, ID=u'mouse')
+        super(Mouse, self).__init__(0, 0, 0, ID='mouse')
 
     def event_collision(self, other):
         sge.game.event_mouse_collision(other)
@@ -1502,6 +1533,17 @@ class Mouse(StellarClass):
         else:
             pygame.mouse.set_visible(False)
 
+    def project_cursor(self):
+        if (not sge.game.grab_input and self.visible and
+                self.sprite is not None):
+            x = (self.mouse_x / sge.game._xscale) - self.sprite.origin_x
+            y = (self.mouse_y / sge.game._yscale) - self.sprite.origin_y
+            img = self.sprite._get_image(
+                self.image_index, self.image_xscale, self.image_yscale, 
+                self.image_rotation, self.image_alpha, self.image_blend)
+
+            sge.game._window_projections.append((img, x, y, None))
+
     def _reset(self):
         # Reset the mouse back to its original state, but NOT position.
         self.z = self._start_z
@@ -1614,8 +1656,8 @@ class _PygameSprite(pygame.sprite.DirtySprite):
             for view in views:
                 x = real_x - view.x - origin_x + view.xport
                 y = real_y - view.y - origin_y + view.yport
-                w = max(1, self.image.get_width())
-                h = max(1, self.image.get_height())
+                w = sprite.width
+                h = sprite.height
                 new_rect = self.image.get_rect()
                 new_rect.left = (round(x * sge.game._xscale) - self.x_offset +
                                  sge.game._x)
@@ -1625,10 +1667,10 @@ class _PygameSprite(pygame.sprite.DirtySprite):
                                x + w <= view.xport + view.width and
                                y >= view.yport and
                                y + h <= view.yport + view.height)
-                within_view = (x + w >= view.xport and
-                               x <= view.xport + view.width and
-                               y + h >= view.yport and
-                               y <= view.yport + view.height)
+                within_view = (x + w > view.xport and
+                               x < view.xport + view.width and
+                               y + h > view.yport and
+                               y < view.yport + view.height)
 
                 if not original_used and inside_view:
                     original_used = True
@@ -1663,15 +1705,17 @@ class _PygameSprite(pygame.sprite.DirtySprite):
                         if y + h > view.yport + view.height:
                             h -= (y + h) - (view.yport + view.height)
 
-                        x = round(x * sge.game._xscale) - self.x_offset
-                        y = round(y * sge.game._yscale) - self.y_offset
-                        cut_x = round(cut_x * sge.game._xscale)
-                        cut_y = round(cut_y * sge.game._yscale)
+                        x = (sge.game._x +
+                             round((x - self.x_offset) * sge.game._xscale))
+                        y = (sge.game._y +
+                             round((y - self.y_offset) * sge.game._yscale))
+                        cut_x *= sge.game._xscale
+                        cut_y *= sge.game._yscale
                         w = round(w * sge.game._xscale)
                         h = round(h * sge.game._yscale)
+                        rect = pygame.Rect(x, y, w, h)
                         cut_rect = pygame.Rect(cut_x, cut_y, w, h)
                         img = self.image.subsurface(cut_rect)
-                        rect = pygame.Rect(x, y, w, h)
 
                     # Create proxy one-time sprite
                     proxy = _PygameOneTimeSprite(img, rect)
@@ -1682,6 +1726,31 @@ class _PygameSprite(pygame.sprite.DirtySprite):
                 self.image.set_colorkey((0, 0, 0))
 
 
+class _PygameProjectionSprite(_PygameSprite):
+
+    # Special Pygame sprite used for room projections.
+
+    def __init__(self, x, y, z, sprite, image_index=0):
+        pygame.sprite.DirtySprite.__init__(self)
+        self.dirty = 1
+        self.x = x
+        self.y = y
+        self.z = z
+        self.sprite = sprite
+        self.image_index = image_index
+        self.image = sprite._get_image(image_index)
+        self.rect = self.image.get_rect()
+        self.x_offset = 0
+        self.y_offset = 0
+
+    def update(self):
+        if self.dirty:
+            self.image = self.sprite._get_image(self.image_index)
+            self.update_rect(self.x, self.y, self.z, self.sprite, 50, 50)
+        else:
+            self.kill()
+
+
 class _FakePygameSprite(_PygameSprite):
 
     # Fake Pygame sprite for "use" by the mouse.
