@@ -15,7 +15,7 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with the Pygame SGE.  If not, see <http://www.gnu.org/licenses/>.
 
-u"""
+"""
 The Stellar Game Engine (abbreviated "SGE", pronounced as "Sage") is a
 free 2-D game engine.  The purpose of the SGE is to make game
 development easier, which allows more rapid development by experienced
@@ -340,7 +340,7 @@ along with the Pygame SGE.  If not, see <http://www.gnu.org/licenses/>.
 Dependencies
 ------------
 
-- Python 3.0 or later <http://www.python.org>
+- Python 3.1 or later <http://www.python.org>
 - Pygame 1.9.2 or later <http://pygame.org>
 
 Formats Support
@@ -376,14 +376,14 @@ For starting position in MOD files, the pattern order number is used
 instead of the number of milliseconds.
 
 If Pygame is built without full image support, :class:`sge.Sprite` will
-only support uncompressed BMP images.  In addition, the pygame.mixer
-module, which is used for audio playback, is optional and depends on
-SDL_mixer; if pygame.mixer is unavailable, sounds and music will not
-play.  If you encounter problems with loading images or playing sounds,
-check your build of Pygame.
+only be able to load uncompressed BMP images.
 
-On some systems, the game will crash if :class:`sge.Music` attempts to
-load an unsupported format.  Since MP3 support is limited, it is best to
+The pygame.mixer module, which is used for all audio playback, is
+optional and depends on SDL_mixer; if pygame.mixer is unavailable,
+sounds and music will not play.
+
+On some systems, :class:`sge.Music` attempting to load an unsupported
+format can crash the game.  Since MP3 support is limited, it is best to
 avoid using it; consider using Ogg Vorbis instead.
 
 Missing Features
@@ -408,17 +408,9 @@ set :data:`sge.hardware_rendering` to :const:`True`.  The benefit of
 hardware acceleration is usually negligible, which is why it is disabled
 by default.
 
-Projection methods are highly inefficient, so they should be avoided if
-speed is important; use the :class:`sge.Sprite` draw methods instead.
-
 """
 
-from __future__ import division
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
-__version__ = u"0.7.0"
+__version__ = "0.8.0"
 
 import os
 
@@ -432,7 +424,7 @@ from .Background import Background
 from .Font import Font
 from .Sound import Sound
 from .Music import Music
-from .StellarClass import StellarClass, Mouse
+from .StellarClass import StellarClass, Mouse, _PygameProjectionSprite
 from .Room import Room
 from .View import View
 from .functions import *
@@ -440,41 +432,31 @@ from .functions import *
 
 __all__ = [
     # Constants
-    u'IMPLEMENTATION', u'ALIGN_LEFT', u'ALIGN_CENTER', u'ALIGN_RIGHT', u'ALIGN_TOP',
-    u'ALIGN_MIDDLE', u'ALIGN_BOTTOM',
+    'IMPLEMENTATION', 'ALIGN_LEFT', 'ALIGN_CENTER', 'ALIGN_RIGHT', 'ALIGN_TOP',
+    'ALIGN_MIDDLE', 'ALIGN_BOTTOM',
 
     # Classes
-    u'Game', u'Sprite', u'BackgroundLayer', u'Background', u'Font', u'Sound',
-    u'Music', u'StellarClass', u'Room', u'View',
+    'Game', 'Sprite', 'BackgroundLayer', 'Background', 'Font', 'Sound',
+    'Music', 'StellarClass', 'Room', 'View',
 
     # Functions
-    u'create_object', u'sound_stop_all', u'music_clear_queue', u'music_stop_all',
-    u'get_key_pressed', u'get_mouse_button_pressed', u'get_joystick_axis',
-    u'get_joystick_hat', u'get_joystick_button_pressed', u'get_joysticks',
-    u'get_joystick_axes', u'get_joystick_hats', u'get_joystick_buttons'
+    'create_object', 'sound_stop_all', 'music_clear_queue', 'music_stop_all',
+    'get_key_pressed', 'get_mouse_button_pressed', 'get_joystick_axis',
+    'get_joystick_hat', 'get_joystick_button_pressed', 'get_joysticks',
+    'get_joystick_axes', 'get_joystick_hats', 'get_joystick_buttons'
     ]
 
 # Global variables
 game = None
-image_directories = [os.path.join(PROGRAM_DIR, u'data', u'images'),
-                     os.path.join(PROGRAM_DIR, u'data', u'sprites'),
-                     os.path.join(PROGRAM_DIR, u'data', u'backgrounds')]
-font_directories = [os.path.join(PROGRAM_DIR, u'data', u'fonts')]
-sound_directories = [os.path.join(PROGRAM_DIR, u'data', u'sounds')]
-music_directories = [os.path.join(PROGRAM_DIR, u'data', u'music')]
+image_directories = [os.path.join(PROGRAM_DIR, 'data', 'images'),
+                     os.path.join(PROGRAM_DIR, 'data', 'sprites'),
+                     os.path.join(PROGRAM_DIR, 'data', 'backgrounds')]
+font_directories = [os.path.join(PROGRAM_DIR, 'data', 'fonts')]
+sound_directories = [os.path.join(PROGRAM_DIR, 'data', 'sounds')]
+music_directories = [os.path.join(PROGRAM_DIR, 'data', 'music')]
 
 hardware_rendering = False
 
-if DEBUG:
-    print(u"Image directories set to:")
-    print(*image_directories, sep=u"; ")
-    print(u"Font directories set to:")
-    print(*font_directories, sep=u"; ")
-    print(u"Sound directories set to:")
-    print(*sound_directories, sep=u"; ")
-    print(u"Music directories set to:")
-    print(*music_directories, sep=u"; ")
-
 # Uncomment this line to tell SDL to center the window.  Disabled by
 # default because it seems to cause some weird behavior with window
 # resizing on at least some systems.
