@@ -15,11 +15,6 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with the Pygame SGE.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import division
-from __future__ import absolute_import
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import os
 import math
 
@@ -28,16 +23,16 @@ import pygame
 import sge
 
 
-__all__ = [u'show_message', u'get_text_entry', u'get_key_pressed',
-           u'get_mouse_button_pressed', u'get_joystick_axis', u'get_joystick_hat',
-           u'get_joystick_button_pressed', u'get_joysticks', u'get_joystick_name',
-           u'get_joystick_id', u'get_joystick_axes', u'get_joystick_hats',
-           u'get_joystick_trackballs', u'get_joystick_buttons', u'_scale',
-           u'_get_pygame_color', u'_scold_user_on_lose_vs_loose']
+__all__ = ['show_message', 'get_text_entry', 'get_key_pressed',
+           'get_mouse_button_pressed', 'get_joystick_axis', 'get_joystick_hat',
+           'get_joystick_button_pressed', 'get_joysticks', 'get_joystick_name',
+           'get_joystick_id', 'get_joystick_axes', 'get_joystick_hats',
+           'get_joystick_trackballs', 'get_joystick_buttons', '_scale',
+           '_get_pygame_color', '_scold_user_on_lose_vs_loose']
 
 
-def show_message(text, buttons=(u'OK',), default=0):
-    u"""Show a dialog box and return the button pressed.
+def show_message(text, buttons=('OK',), default=0):
+    """Show a dialog box and return the button pressed.
 
     Arguments:
 
@@ -58,8 +53,8 @@ def show_message(text, buttons=(u'OK',), default=0):
     return _show_modal(text, default, False, buttons)
 
 
-def get_text_entry(text, default=u''):
-    u"""Show a text entry dialog box and return the text entered.
+def get_text_entry(text, default=''):
+    """Show a text entry dialog box and return the text entered.
 
     Arguments:
 
@@ -76,13 +71,13 @@ def get_text_entry(text, default=u''):
 
     """
     pygame.key.set_repeat(500, 20)
-    text = _show_modal(text, default, True, (u'Cancel', u'OK'))
+    text = _show_modal(text, default, True, ('Cancel', 'OK'))
     pygame.key.set_repeat()
     return text
 
 
 def get_key_pressed(key):
-    u"""Return whether or not a key is pressed.
+    """Return whether or not a key is pressed.
 
     Arguments:
 
@@ -228,7 +223,7 @@ def get_key_pressed(key):
 
 
 def get_mouse_button_pressed(button):
-    u"""Return whether or not a mouse button is pressed.
+    """Return whether or not a mouse button is pressed.
 
     Arguments:
 
@@ -248,7 +243,7 @@ def get_mouse_button_pressed(button):
     ====================== =================
 
     """
-    b = {u"left": 0, u"middle": 1, u"right": 2}.setdefault(button.lower())
+    b = {"left": 0, "middle": 1, "right": 2}.setdefault(button.lower())
     if b is not None:
         return pygame.mouse.get_pressed()[b]
     else:
@@ -256,7 +251,7 @@ def get_mouse_button_pressed(button):
 
 
 def get_joystick_axis(joystick, axis):
-    u"""Return the position of a joystick axis.
+    """Return the position of a joystick axis.
 
     Arguments:
 
@@ -281,7 +276,7 @@ def get_joystick_axis(joystick, axis):
 
 
 def get_joystick_hat(joystick, hat):
-    u"""Return the position of a joystick HAT.
+    """Return the position of a joystick HAT.
 
     Arguments:
 
@@ -306,7 +301,7 @@ def get_joystick_hat(joystick, hat):
 
 
 def get_joystick_button_pressed(joystick, button):
-    u"""Return whether or not a joystick button is pressed.
+    """Return whether or not a joystick button is pressed.
 
     Arguments:
 
@@ -333,12 +328,12 @@ def get_joystick_button_pressed(joystick, button):
 
 
 def get_joysticks():
-    u"""Return the number of joysticks available."""
+    """Return the number of joysticks available."""
     return len(sge.game._joysticks)
 
 
 def get_joystick_name(joystick):
-    u"""Return the name of a joystick.
+    """Return the name of a joystick.
 
     Arguments:
 
@@ -357,7 +352,7 @@ def get_joystick_name(joystick):
 
 
 def get_joystick_id(joystick):
-    u"""Return the number of a joystick, where ``0`` is the first joystick.
+    """Return the number of a joystick, where ``0`` is the first joystick.
 
     Arguments:
 
@@ -376,7 +371,7 @@ def get_joystick_id(joystick):
 
 
 def get_joystick_axes(joystick):
-    u"""Return the number of axes available on a joystick.
+    """Return the number of axes available on a joystick.
 
     Arguments:
 
@@ -395,7 +390,7 @@ def get_joystick_axes(joystick):
 
 
 def get_joystick_hats(joystick):
-    u"""Return the number of HATs available on a joystick.
+    """Return the number of HATs available on a joystick.
 
     Arguments:
 
@@ -414,7 +409,7 @@ def get_joystick_hats(joystick):
 
 
 def get_joystick_trackballs(joystick):
-    u"""Return the number of trackballs available on a joystick.
+    """Return the number of trackballs available on a joystick.
 
     Arguments:
 
@@ -433,7 +428,7 @@ def get_joystick_trackballs(joystick):
 
 
 def get_joystick_buttons(joystick):
-    u"""Return the number of buttons available on a joystick.
+    """Return the number of buttons available on a joystick.
 
     Arguments:
 
@@ -462,7 +457,7 @@ def _show_modal(text, default, text_entry, buttons):
     screenshot = window.copy()
     background = screenshot.copy()
     sge.font_directories.append(os.path.dirname(__file__))
-    font = sge.Font(u"DroidSans-Bold.ttf", size=12)
+    font = sge.Font("DroidSans-Bold.ttf", size=12)
     del sge.font_directories[-1]
     window_rect = window.get_rect()
     screen_w, screen_h = background.get_size()
@@ -493,96 +488,96 @@ def _show_modal(text, default, text_entry, buttons):
     mydir = os.path.dirname(__file__)
     try:
         box_fill = pygame.image.load(
-            os.path.join(mydir, u'sge_dialogbox.png')).convert_alpha()
+            os.path.join(mydir, 'sge_dialogbox.png')).convert_alpha()
         box_left = pygame.image.load(
-            os.path.join(mydir, u'sge_dialogbox_left.png')).convert_alpha()
+            os.path.join(mydir, 'sge_dialogbox_left.png')).convert_alpha()
         box_right = pygame.image.load(
-            os.path.join(mydir, u'sge_dialogbox_right.png')).convert_alpha()
+            os.path.join(mydir, 'sge_dialogbox_right.png')).convert_alpha()
         box_top = pygame.image.load(
-            os.path.join(mydir, u'sge_dialogbox_top.png')).convert_alpha()
+            os.path.join(mydir, 'sge_dialogbox_top.png')).convert_alpha()
         box_bottom = pygame.image.load(
-            os.path.join(mydir, u'sge_dialogbox_bottom.png')).convert_alpha()
+            os.path.join(mydir, 'sge_dialogbox_bottom.png')).convert_alpha()
         box_topleft = pygame.image.load(
-            os.path.join(mydir, u'sge_dialogbox_topleft.png')).convert_alpha()
+            os.path.join(mydir, 'sge_dialogbox_topleft.png')).convert_alpha()
         box_topright = pygame.image.load(
-            os.path.join(mydir, u'sge_dialogbox_topright.png')).convert_alpha()
+            os.path.join(mydir, 'sge_dialogbox_topright.png')).convert_alpha()
         box_bottomleft = pygame.image.load(os.path.join(
-            mydir, u'sge_dialogbox_bottomleft.png')).convert_alpha()
+            mydir, 'sge_dialogbox_bottomleft.png')).convert_alpha()
         box_bottomright = pygame.image.load(os.path.join(
-            mydir, u'sge_dialogbox_bottomright.png')).convert_alpha()
+            mydir, 'sge_dialogbox_bottomright.png')).convert_alpha()
         text_entry_fill = pygame.image.load(
-            os.path.join(mydir, u'sge_text_entry.png')).convert_alpha()
+            os.path.join(mydir, 'sge_text_entry.png')).convert_alpha()
         text_entry_left = pygame.image.load(
-            os.path.join(mydir, u'sge_text_entry_left.png')).convert_alpha()
+            os.path.join(mydir, 'sge_text_entry_left.png')).convert_alpha()
         text_entry_right = pygame.image.load(
-            os.path.join(mydir, u'sge_text_entry_right.png')).convert_alpha()
+            os.path.join(mydir, 'sge_text_entry_right.png')).convert_alpha()
         text_entry_top = pygame.image.load(
-            os.path.join(mydir, u'sge_text_entry_top.png')).convert_alpha()
+            os.path.join(mydir, 'sge_text_entry_top.png')).convert_alpha()
         text_entry_bottom = pygame.image.load(
-            os.path.join(mydir, u'sge_text_entry_bottom.png')).convert_alpha()
+            os.path.join(mydir, 'sge_text_entry_bottom.png')).convert_alpha()
         text_entry_topleft = pygame.image.load(
-            os.path.join(mydir, u'sge_text_entry_topleft.png')).convert_alpha()
+            os.path.join(mydir, 'sge_text_entry_topleft.png')).convert_alpha()
         text_entry_topright = pygame.image.load(
-            os.path.join(mydir, u'sge_text_entry_topright.png')).convert_alpha()
+            os.path.join(mydir, 'sge_text_entry_topright.png')).convert_alpha()
         text_entry_bottomleft = pygame.image.load(os.path.join(
-            mydir, u'sge_text_entry_bottomleft.png')).convert_alpha()
+            mydir, 'sge_text_entry_bottomleft.png')).convert_alpha()
         text_entry_bottomright = pygame.image.load(os.path.join(
-            mydir, u'sge_text_entry_bottomright.png')).convert_alpha()
+            mydir, 'sge_text_entry_bottomright.png')).convert_alpha()
         button_fill = pygame.image.load(
-            os.path.join(mydir, u'sge_button.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button.png')).convert_alpha()
         button_left = pygame.image.load(
-            os.path.join(mydir, u'sge_button_left.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_left.png')).convert_alpha()
         button_right = pygame.image.load(
-            os.path.join(mydir, u'sge_button_right.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_right.png')).convert_alpha()
         button_top = pygame.image.load(
-            os.path.join(mydir, u'sge_button_top.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_top.png')).convert_alpha()
         button_bottom = pygame.image.load(
-            os.path.join(mydir, u'sge_button_bottom.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_bottom.png')).convert_alpha()
         button_topleft = pygame.image.load(
-            os.path.join(mydir, u'sge_button_topleft.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_topleft.png')).convert_alpha()
         button_topright = pygame.image.load(
-            os.path.join(mydir, u'sge_button_topright.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_topright.png')).convert_alpha()
         button_bottomleft = pygame.image.load(
-            os.path.join(mydir, u'sge_button_bottomleft.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_bottomleft.png')).convert_alpha()
         button_bottomright = pygame.image.load(
-            os.path.join(mydir, u'sge_button_bottomright.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_bottomright.png')).convert_alpha()
         button_selected_fill = pygame.image.load(
-            os.path.join(mydir, u'sge_button_selected.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_selected.png')).convert_alpha()
         button_selected_left = pygame.image.load(os.path.join(
-            mydir, u'sge_button_selected_left.png')).convert_alpha()
+            mydir, 'sge_button_selected_left.png')).convert_alpha()
         button_selected_right = pygame.image.load(os.path.join(
-            mydir, u'sge_button_selected_right.png')).convert_alpha()
+            mydir, 'sge_button_selected_right.png')).convert_alpha()
         button_selected_top = pygame.image.load(
-            os.path.join(mydir, u'sge_button_selected_top.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_selected_top.png')).convert_alpha()
         button_selected_bottom = pygame.image.load(os.path.join(
-            mydir, u'sge_button_selected_bottom.png')).convert_alpha()
+            mydir, 'sge_button_selected_bottom.png')).convert_alpha()
         button_selected_topleft = pygame.image.load(os.path.join(
-            mydir, u'sge_button_selected_topleft.png')).convert_alpha()
+            mydir, 'sge_button_selected_topleft.png')).convert_alpha()
         button_selected_topright = pygame.image.load(os.path.join(
-            mydir, u'sge_button_selected_topright.png')).convert_alpha()
+            mydir, 'sge_button_selected_topright.png')).convert_alpha()
         button_selected_bottomleft = pygame.image.load(os.path.join(
-            mydir, u'sge_button_selected_bottomleft.png')).convert_alpha()
+            mydir, 'sge_button_selected_bottomleft.png')).convert_alpha()
         button_selected_bottomright = pygame.image.load(os.path.join(
-            mydir, u'sge_button_selected_bottomright.png')).convert_alpha()
+            mydir, 'sge_button_selected_bottomright.png')).convert_alpha()
         button_press_fill = pygame.image.load(
-            os.path.join(mydir, u'sge_button_press.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_press.png')).convert_alpha()
         button_press_left = pygame.image.load(
-            os.path.join(mydir, u'sge_button_press_left.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_press_left.png')).convert_alpha()
         button_press_right = pygame.image.load(
-            os.path.join(mydir, u'sge_button_press_right.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_press_right.png')).convert_alpha()
         button_press_top = pygame.image.load(
-            os.path.join(mydir, u'sge_button_press_top.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_press_top.png')).convert_alpha()
         button_press_bottom = pygame.image.load(
-            os.path.join(mydir, u'sge_button_press_bottom.png')).convert_alpha()
+            os.path.join(mydir, 'sge_button_press_bottom.png')).convert_alpha()
         button_press_topleft = pygame.image.load(os.path.join(
-            mydir, u'sge_button_press_topleft.png')).convert_alpha()
+            mydir, 'sge_button_press_topleft.png')).convert_alpha()
         button_press_topright = pygame.image.load(os.path.join(
-            mydir, u'sge_button_press_topright.png')).convert_alpha()
+            mydir, 'sge_button_press_topright.png')).convert_alpha()
         button_press_bottomleft = pygame.image.load(os.path.join(
-            mydir, u'sge_button_press_bottomleft.png')).convert_alpha()
+            mydir, 'sge_button_press_bottomleft.png')).convert_alpha()
         button_press_bottomright = pygame.image.load(os.path.join(
-            mydir, u'sge_button_press_bottomright.png')).convert_alpha()
-    except pygame.error, e:
+            mydir, 'sge_button_press_bottomright.png')).convert_alpha()
+    except pygame.error as e:
         if sge.DEBUG:
             print(e)
         box_fill = pygame.Surface((1, 1))
@@ -638,8 +633,8 @@ def _show_modal(text, default, text_entry, buttons):
 
     # Box image
     box = pygame.Surface((box_w, box_h), pygame.SRCALPHA)
-    for i in xrange(0, box_w, box_fill.get_width()):
-        for j in xrange(0, box_h, box_fill.get_height()):
+    for i in range(0, box_w, box_fill.get_width()):
+        for j in range(0, box_h, box_fill.get_height()):
             box.blit(box_fill, (i, j))
 
     # Clear the way for the corners and edges (so transparency works
@@ -661,18 +656,18 @@ def _show_modal(text, default, text_entry, buttons):
     box.blit(box_bottomleft, (0, box_h - box_bottomleft.get_height()))
     box.blit(box_bottomright, (box_w - box_bottomright.get_width(),
                                box_h - box_bottomright.get_height()))
-    for i in xrange(box_topleft.get_width(), box_w - box_topright.get_width(),
+    for i in range(box_topleft.get_width(), box_w - box_topright.get_width(),
                     box_top.get_width()):
         box.blit(box_top, (i, 0))
-    for i in xrange(box_bottomleft.get_width(),
+    for i in range(box_bottomleft.get_width(),
                     box_w - box_bottomright.get_width(),
                     box_bottom.get_width()):
         box.blit(box_bottom, (i, box_h - box_bottom.get_height()))
-    for i in xrange(box_topleft.get_height(),
+    for i in range(box_topleft.get_height(),
                     box_h - box_bottomleft.get_height(),
                     box_left.get_height()):
         box.blit(box_left, (0, i))
-    for i in xrange(box_topright.get_height(),
+    for i in range(box_topright.get_height(),
                     box_h - box_bottomright.get_height(),
                     box_right.get_height()):
         box.blit(box_right, (box_w - box_right.get_width(), i))
@@ -681,8 +676,8 @@ def _show_modal(text, default, text_entry, buttons):
     # Text Entry image
     text_entry_field = pygame.Surface((text_entry_w, text_entry_h),
                                       pygame.SRCALPHA)
-    for i in xrange(0, text_entry_w, text_entry_fill.get_width()):
-        for j in xrange(0, text_entry_h, text_entry_fill.get_height()):
+    for i in range(0, text_entry_w, text_entry_fill.get_width()):
+        for j in range(0, text_entry_h, text_entry_fill.get_height()):
             text_entry_field.blit(text_entry_fill, (i, j))
 
     # Clear the way for the corners and edges (so transparency works
@@ -708,20 +703,20 @@ def _show_modal(text, default, text_entry, buttons):
     text_entry_field.blit(text_entry_bottomright,
                           (text_entry_w - text_entry_bottomright.get_width(),
                            text_entry_h - text_entry_bottomright.get_height()))
-    for i in xrange(text_entry_topleft.get_width(),
+    for i in range(text_entry_topleft.get_width(),
                     text_entry_w - text_entry_topright.get_width(),
                     text_entry_top.get_width()):
         text_entry_field.blit(text_entry_top, (i, 0))
-    for i in xrange(text_entry_bottomleft.get_width(),
+    for i in range(text_entry_bottomleft.get_width(),
                     text_entry_w - text_entry_bottomright.get_width(),
                     text_entry_bottom.get_width()):
         text_entry_field.blit(text_entry_bottom, (
             i, text_entry_h - text_entry_bottom.get_height()))
-    for i in xrange(text_entry_topleft.get_height(),
+    for i in range(text_entry_topleft.get_height(),
                     text_entry_h - text_entry_bottomleft.get_height(),
                     text_entry_left.get_height()):
         text_entry_field.blit(text_entry_left, (0, i))
-    for i in xrange(text_entry_topright.get_height(),
+    for i in range(text_entry_topright.get_height(),
                     text_entry_h - text_entry_bottomright.get_height(),
                     text_entry_right.get_height()):
         text_entry_field.blit(text_entry_right, (
@@ -739,8 +734,8 @@ def _show_modal(text, default, text_entry, buttons):
 
     # Button image
     button = pygame.Surface((button_w, button_h), pygame.SRCALPHA)
-    for i in xrange(0, button_w, button_fill.get_width()):
-        for j in xrange(0, button_h, button_fill.get_height()):
+    for i in range(0, button_w, button_fill.get_width()):
+        for j in range(0, button_h, button_fill.get_height()):
             button.blit(button_fill, (i, j))
 
     # Clear the way for the corners and edges (so transparency works
@@ -765,27 +760,27 @@ def _show_modal(text, default, text_entry, buttons):
     button.blit(button_bottomright,
                 (button_w - button_bottomright.get_width(),
                  button_h - button_bottomright.get_height()))
-    for i in xrange(button_topleft.get_width(),
+    for i in range(button_topleft.get_width(),
                     button_w - button_topright.get_width(),
                     button_top.get_width()):
         button.blit(button_top, (i, 0))
-    for i in xrange(button_bottomleft.get_width(),
+    for i in range(button_bottomleft.get_width(),
                     button_w - button_bottomright.get_width(),
                     button_bottom.get_width()):
         button.blit(button_bottom, (i, button_h - button_bottom.get_height()))
-    for i in xrange(button_topleft.get_height(),
+    for i in range(button_topleft.get_height(),
                     button_h - button_bottomleft.get_height(),
                     button_left.get_height()):
         button.blit(button_left, (0, i))
-    for i in xrange(button_topright.get_height(),
+    for i in range(button_topright.get_height(),
                     button_h - button_bottomright.get_height(),
                     button_right.get_height()):
         button.blit(button_right, (button_w - button_right.get_width(), i))
 
     # Button image when selected
     button_selected = pygame.Surface((button_w, button_h), pygame.SRCALPHA)
-    for i in xrange(0, button_w, button_selected_fill.get_width()):
-        for j in xrange(0, button_h, button_selected_fill.get_height()):
+    for i in range(0, button_w, button_selected_fill.get_width()):
+        for j in range(0, button_h, button_selected_fill.get_height()):
             button_selected.blit(button_selected_fill, (i, j))
 
     # Clear the way for the corners and edges (so transparency works
@@ -812,21 +807,21 @@ def _show_modal(text, default, text_entry, buttons):
     button_selected.blit(button_selected_bottomright,
                          (button_w - button_selected_bottomright.get_width(),
                           button_h - button_selected_bottomright.get_height()))
-    for i in xrange(button_selected_topleft.get_width(),
+    for i in range(button_selected_topleft.get_width(),
                     button_w - button_selected_topright.get_width(),
                     button_selected_top.get_width()):
         button_selected.blit(button_selected_top, (i, 0))
-    for i in xrange(button_selected_bottomleft.get_width(),
+    for i in range(button_selected_bottomleft.get_width(),
                     button_w - button_selected_bottomright.get_width(),
                     button_selected_bottom.get_width()):
         button_selected.blit(
             button_selected_bottom,
             (i, button_h - button_selected_bottom.get_height()))
-    for i in xrange(button_selected_topleft.get_height(),
+    for i in range(button_selected_topleft.get_height(),
                     button_h - button_selected_bottomleft.get_height(),
                     button_selected_left.get_height()):
         button_selected.blit(button_selected_left, (0, i))
-    for i in xrange(button_selected_topright.get_height(),
+    for i in range(button_selected_topright.get_height(),
                     button_h - button_selected_bottomright.get_height(),
                     button_selected_right.get_height()):
         button_selected.blit(
@@ -835,8 +830,8 @@ def _show_modal(text, default, text_entry, buttons):
 
     # Button image when pressed
     button_press = pygame.Surface((button_w, button_h), pygame.SRCALPHA)
-    for i in xrange(0, button_w, button_press_fill.get_width()):
-        for j in xrange(0, button_h, button_press_fill.get_height()):
+    for i in range(0, button_w, button_press_fill.get_width()):
+        for j in range(0, button_h, button_press_fill.get_height()):
             button_press.blit(button_press_fill, (i, j))
 
     # Clear the way for the corners and edges (so transparency works
@@ -862,27 +857,27 @@ def _show_modal(text, default, text_entry, buttons):
     button_press.blit(button_press_bottomright,
                       (button_w - button_press_bottomright.get_width(),
                        button_h - button_press_bottomright.get_height()))
-    for i in xrange(button_press_topleft.get_width(),
+    for i in range(button_press_topleft.get_width(),
                     button_w - button_press_topright.get_width(),
                     button_press_top.get_width()):
         button_press.blit(button_press_top, (i, 0))
-    for i in xrange(button_press_bottomleft.get_width(),
+    for i in range(button_press_bottomleft.get_width(),
                     button_w - button_press_bottomright.get_width(),
                     button_press_bottom.get_width()):
         button_press.blit(button_press_bottom,
                           (i, button_h - button_press_bottom.get_height()))
-    for i in xrange(button_press_topleft.get_height(),
+    for i in range(button_press_topleft.get_height(),
                     button_h - button_press_bottomleft.get_height(),
                     button_press_left.get_height()):
         button_press.blit(button_press_left, (0, i))
-    for i in xrange(button_press_topright.get_height(),
+    for i in range(button_press_topright.get_height(),
                     button_h - button_press_bottomright.get_height(),
                     button_press_right.get_height()):
         button_press.blit(button_press_right,
                           (button_w - button_press_right.get_width(), i))
 
     message_sprite = sge.Sprite(width=text_w, height=text_h)
-    message_sprite.draw_text(font, text, 0, 0, text_w, text_h, color=u'black')
+    message_sprite.draw_text(font, text, 0, 0, text_w, text_h, color='black')
     box.blit(message_sprite._baseimages[0], (4, 4))
     del sge.game.sprites[message_sprite.id]
 
@@ -890,7 +885,7 @@ def _show_modal(text, default, text_entry, buttons):
     press_buttons = []
     button_rects = []
     button_y = box_h - 8
-    for i in xrange(len(buttons)):
+    for i in range(len(buttons)):
         button_surf = button.copy()
         button_selected_surf = button_selected.copy()
         button_press_surf = button_press.copy()
@@ -920,12 +915,12 @@ def _show_modal(text, default, text_entry, buttons):
         selection = 1
     else:
         selection = default
-        text_entered = u""
+        text_entered = ""
         cursor_position = 0
     prev_axis_value = []
-    for j in xrange(get_joysticks()):
+    for j in range(get_joysticks()):
         axes = []
-        for a in xrange(get_joystick_axes(j)):
+        for a in range(get_joystick_axes(j)):
             axes.append(0)
         prev_axis_value.append(axes)
     button_entered = None
@@ -966,13 +961,13 @@ def _show_modal(text, default, text_entry, buttons):
                     if text_entry and cursor_position > 0:
                         text_entered_list = list(text_entered)
                         del text_entered_list[cursor_position - 1]
-                        text_entered = u''.join(text_entered_list)
+                        text_entered = ''.join(text_entered_list)
                         cursor_position -= 1
                 elif event.key == pygame.K_DELETE:
                     if text_entry and cursor_position < len(text_entered):
                         text_entered_list = list(text_entered)
                         del text_entered_list[cursor_position]
-                        text_entered = u''.join(text_entered_list)
+                        text_entered = ''.join(text_entered_list)
                 elif event.key == pygame.K_ESCAPE:
                     _finish_modal()
                     return
@@ -981,7 +976,7 @@ def _show_modal(text, default, text_entry, buttons):
                         text_entered_list = list(text_entered)
                         text_entered_list.insert(cursor_position,
                                                  event.unicode)
-                        text_entered = u''.join(text_entered_list)
+                        text_entered = ''.join(text_entered_list)
                         cursor_position += 1
             elif event.type == pygame.KEYUP:
                 if (event.key == pygame.K_RETURN or
@@ -1046,23 +1041,23 @@ def _show_modal(text, default, text_entry, buttons):
                     button_entered = None
             elif event.type == pygame.MOUSEMOTION:
                 x, y = event.pos
-                for i in xrange(len(button_rects)):
+                for i in range(len(button_rects)):
                     rect = button_rects[i]
                     if (rect.left <= x <= rect.right and
                             rect.top <= y <= rect.bottom):
                         selection = i
                         break
             elif event.type == pygame.MOUSEBUTTONDOWN:
-                if event.button == sge.MOUSE_BUTTONS[u'left']:
+                if event.button == sge.MOUSE_BUTTONS['left']:
                     x, y = event.pos
-                    for i in xrange(len(button_rects)):
+                    for i in range(len(button_rects)):
                         rect = button_rects[i]
                         if (rect.left <= x <= rect.right and
                                 rect.top <= y <= rect.bottom):
                             button_clicked = i
                             break
             elif event.type == pygame.MOUSEBUTTONUP:
-                if (event.button == sge.MOUSE_BUTTONS[u'left'] and
+                if (event.button == sge.MOUSE_BUTTONS['left'] and
                         button_clicked is not None):
                     x, y = event.pos
                     rect = button_rects[button_clicked]
@@ -1082,13 +1077,13 @@ def _show_modal(text, default, text_entry, buttons):
                         button_clicked = None
             elif event.type == pygame.QUIT:
                 if sge.DEBUG:
-                    print(u'Quit requested by the system.')
+                    print('Quit requested by the system.')
                 pygame.event.post(event)
                 _finish_modal()
                 return
             elif event.type == pygame.VIDEORESIZE:
                 if sge.DEBUG:
-                    print(u'Video resize detected.')
+                    print('Video resize detected.')
                 sge.game._window_width = event.w
                 sge.game._window_height = event.h
                 sge.game._set_mode()
@@ -1159,12 +1154,12 @@ def _scale(surface, width, height):
 
 def _get_pygame_color(color):
     # Return the proper Pygame color.
-    if isinstance(color, basestring):
+    if isinstance(color, str):
         c = color.lower()
         if c in sge.COLORS:
             c = sge.COLORS[c]
 
-        return pygame.Color(bytes(c))
+        return pygame.Color(c)
     elif isinstance(color, int):
         r = int((color & 0xff0000) // (256 ** 2))
         g = int((color & 0x00ff00) // 256)
@@ -1181,12 +1176,12 @@ def _get_pygame_color(color):
 
 def _scold_user_on_lose_vs_loose(attempted_name):
     # Tell the user that they misspelled "lose" as "loose".
-    m = u'\n'.join((
-        u'Huh? I don\'t have a method called "{0}".'.format(attempted_name),
-        u'You do know that "lose" (a verb meaning "to fail to keep or hold")',
-        u'is not spelled the same as "loose" (an adjective meaning "not',
-        u'tightly fastened, attached, or held"), right?'))
+    m = '\n'.join((
+        'Huh? I don\'t have a method called "{}".'.format(attempted_name),
+        'You do know that "lose" (a verb meaning "to fail to keep or hold")',
+        'is not spelled the same as "loose" (an adjective meaning "not',
+        'tightly fastened, attached, or held"), right?'))
 
     print(m)
-    m = m.replace(u'\n', u' ')
+    m = m.replace('\n', ' ')
     show_message(m)
